/*
	csclok.c
	
	12/90 pmcm

	% userwin clock portion of the C-scape Demo.

	C-scape 4.0 Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	This program relies on OGL--the Oakland Graphics Library--to draw a 
	clock in a userwin.  The clock appears in color undera graphics
	display mode that has at least 16 available colors; else, it appears
	in monochrome.

	Comments in the code explain the calculations used to draw the
	clock.

	Revision History:
	-----------------
	12/05/90 pmcm	composed from jmd's ogl and gam's clock scraps
	 1/22/91 pmcm	took cs32clok.c from democlok.c, added dgraf_Close,
					replace hard_Init and sedwin_ClassInit w/ disp_ShowMouse
	 1/22/91 pmcm	added gam's fix for smooth hours
	 1/22/91 pmcm	added keyboard move and resize
	 1/22/91 pmcm	spliced in jmd's digital face
	 1/27/91 pmcm	added help button and help key support
	 1/28/91 pmcm 	moved graphics junk to cs32graf.c, cs32demo.h
	 2/05/91 jmd	fixed 12:00 displayed as 0:00 bug
	 4/03/91 bkd	added "void" to empty argument lists

	12/19/91 pmcm	removed clock_NoInitGrafMsg()
	12/19/91 pmcm	replaced dgraf_ stuff with OGL
	 1/07/92 pmcm	added BD_OUTLINE to clock
	
	 1/27/92 pmcm	changed ogl_ to new names
	 3/06/92 pmcm	changed mouse handlers to winmou_All
	 5/31/92 pmcm	added 0 paint area check to clock_Draw(); PI is OAK_PI.
	 6/15/92 pmcm 	added memory checks.
	 7/06/92 pmcm	added MOU_NOFIELD case to aux_Clock
	 7/20/92 mla	changed sed_Find() to obj_Find()
	 8/25/92 pmcm	fixed init of clockcolor_struct bord attr;
					redid colormap initialization.
*/

#include <time.h>		/* 	for time structure */
#include <math.h>		/* 	for sin() and cos() */

#include "csdemo.h"

#include "kbidle.h"		/* 	for idle function */
#include "cstime.h"		/* 	for C-scape time routines */
#include "helpdecl.h"	/* 	for calling help from userwin */
#include "oakalloc.h"	/*	for omalloc() */
#include "ogldecl.h"
#include "osysdecl.h"

/*----------------------------- CONSTANTS ------------------------------------*/

#define OAK_PI						3.141592

/*	default display window attribute--used to 
	set the display background color 
*/
#define DISP_ATTR		0x10

/*	paints for the color clock */

#define CLOCK_CBACK		11
#define CLOCK_CHAND		1
#define CLOCK_CSEC		4
#define CLOCK_CTICK		0
#define CLOCK_CBORD		0xb3

/*	shadow paints for the color clock */

#define CLOCK_SHADBACK		3
#define CLOCK_SHADHAND		8
#define CLOCK_SHADSEC		4
#define CLOCK_SHADTICK		7
#define CLOCK_SHADBORD		0x3b

/*	paints for monochrome clock.
	(shadows are rendered inverse
	see clock_Draw.)
*/
#define CLOCK_MBACK			1
#define CLOCK_MHAND			0
#define CLOCK_MSEC			0
#define CLOCK_MTICK			0
#define CLOCK_MBORD			0x10

/*----------------------------- MODULE GLOBALS -------------------------------*/
opixval			oblack 	= 0;
opixval			owhite 	= 1;

/*----------------------------- TYPES ----------------------------------------*/
/*	clock color data
	the clock is rendered in color only on displays supporting 16 
	or more colors.  if the display supports less colors, the clock
	is rendered in monochrome.
*/

struct clockcolor_struct {
	int		back;		/*  color of the background */
	int 	tick;		/*  color of the tickmarks around the clock face */
	int		hand;		/*	color of the minute and hour hands */
	int		sec;		/* 	color of the second hand */
	byte	bord;		/*	attribute of the clock window border */
};

/*	clock plot data (see clock_InitData(), below */

struct tick_struct {
	double	x1;
	double	y1;
	double	x2;
	double	y2;
};

/*	clock data structure */

typedef struct _clockdata_struct {
	boolean						mono;			/*	color or mono clock? */
	boolean						analog;			/*	analog or digital clock */
	int							msg;			/*	regular paint or shadow? */
	struct clockcolor_struct 	color_clock;	/*	color paints */
	struct clockcolor_struct 	shadow_clock;	/*	color shadow paints */
	struct clockcolor_struct 	mono_clock; 	/* 	monochrome paints */
	struct tm 					time;			/*	current time */
	struct tick_struct			tick[12];		/* 	tickmark positions */
} clockdata_struct;

typedef clockdata_struct *clockdata_type;

/*----------------------------- PROTOTYPES -----------------------------------*/

aux_func	(aux_Clock);
idle_func	(clock_Idle);

win_type 	clock_Open(void);
void 		clock_InitData(win_type clock);
void 		clock_Draw(ptd_struct *ptd);
void 		clock_DrawHand(ptd_struct *ptd, double theta, double radius, boolean sec, int color);
void 		clock_DrawTicks(ptd_struct *ptd, int color);
void		cdp_GetColors(clockdata_type cdp, int *bck, int *tck, int *hnd, int *sec);
void 		seg7_DrawNum(ptd_struct *ptd, int x, int y, int wid, int hgt, opixval fg, int num);

/* -------------------------------------------------------------------------- */
boolean demo_Clock(void)
{
	boolean ret = TRUE;

	/*	install idle function (creates the clock) */
	kb_Idle(clock_Idle); 

	if (obj_Find("Clock") != NULL) {
		oak_ClearAllocErrorTag();

		/*	create, run and clock the clock prompt window */
		demo_LoadSed("ClockMsg", DL_PAINT | DL_GO | DL_CLOSE | DL_HELPBTN | DL_ESCBTN);

		if (oak_ClearAllocErrorTag() != 0) {
			/*	allocation error during load and run of ClockMsg screen */
			oak_ClearAllocErrorTag();
			ret = FALSE;
		}
	}
	else {
		ret = FALSE;
	}

	/*	remove idle function (destroys the clock) */
	kb_Idle(FNULL); 

	return(ret);
}

/*----------------------------- CLOCK ROUTINES -------------------------------*/
int clock_Idle(int msg, unsigned wait)
/*
	open the clock when this idle function is connected.
	run the clock when we have time.
	close the clock when this idle function is disconnected. 
*/
{
	static win_type 	   	clock; 
	static short 		   	oldsec;
	static clockdata_type	cdp;

	oak_notused(wait);

	switch (msg) {
	
	case IDLE_START:
		/*	build clock (inits its own data) */
		clock = NULL;
		if ((clock = clock_Open()) != NULL) {

			cdp = (clockdata_type) win_GetData(clock);

			/*	paint the clock to the display */
			win_Employ(clock);
		}
		break;

	case IDLE_CHECK:
	case IDLE_READ:
		if (clock != NULL) {

			/* 	get current time */
			tm_Now(&(cdp->time));

			/* 	paint the clock every new second */
			if (oldsec != cdp->time.tm_sec) {
				oldsec = (short)cdp->time.tm_sec;
				win_Paint(clock);
			}
		}
		break;

	case IDLE_STOP:
		/*	close the clock and free its data storage */
		if (clock != NULL) {
			if (cdp != NULL) {
			 	ofree(OA_NOTAG, cdp);
				cdp = NULL;
				win_SetData(clock, NULL);
			}
			win_Close(clock);
			clock = NULL;
		}
		break;

	default:
		break;
	}

	return(KEY_NONE);
}

/* -------------------------------------------------------------------------- */
win_type clock_Open(void)
/*	
	opens the clock window of the userwin_Class.
	sets the clock window size, position, etc.
	initializes the clock data.
	returns the clock window to caller to paint and run.
*/														   
{
	win_type 		clock;
	ocbox 			cbox;
	clockdata_type	cdp;


	/*	set up an Oakland character box for win_Open() */
	cbox.toprow   = 0;
	cbox.leftcol  = 0;
	cbox.botrow   = 1;
	cbox.rightcol = 1;

	/*	open the clock window */
	if ((clock = win_Open(userwin_Class, &cbox)) == NULL) {
	 	return(NULL);
	}

	/*	make clock window area proportional to the display */
	win_SetSize(clock, disp_GetHeight()/3, disp_GetWidth()/3);

	/*	open a border for the window and allocate the clock data storage */
	if (!win_SetBorder(clock, bd_cua)  || 
		(cdp = (clockdata_type) omalloc(OA_NOTAG, 
		sizeof(clockdata_struct))) == NULL) {

		win_Close(clock);
		return(NULL);
	}

	/*	attach the clock data to the clock window's generic data pointer */
	win_SetData(clock, (VOID *)cdp);

	/*	initialize the clock data */
	clock_InitData(clock);

	/*	set some miscellaneous stuff */

	bord_SetFeature(clock, BD_MOVE | BD_RESIZE | BD_TOP | BD_OUTLINE);
	win_SetShadow(clock, 1);
	win_SetShadowAttr(clock, CLOCK_SHADBORD);
	bord_SetAttr(clock, cdp->mono ? cdp->mono_clock.bord : cdp->color_clock.bord);

	/* 	position so there is a char-size space between clock (with border
	 *	and shadow) and display right; and between clock and demo promptbar.
	 */
 	win_SetPosition(clock, (disp_GetHeight() - bord_GetHeight(clock) - 2),
			(disp_GetWidth()  - bord_GetWidth(clock)  - 1));

	win_SetMouse(clock, winmou_All);
	win_SetMouseFeature(clock, MOUF_TRACK);

	obj_SetName(clock, "Clock");
	obj_SetAux(clock, aux_Clock);

 	return(clock);
}

/* -------------------------------------------------------------------------- */
void clock_InitData(win_type clock)
/*
  	initialize the clock data.
*/
{
	clockdata_type	cdp;
	int 			tickno;
	double  		theta;
	double			ourcos, oursin;
	ocolmap_type 	colors;
	byte 			attr;
	opixval 		pv;


	if (clock != NULL && 
		(cdp = (clockdata_type) win_GetData(clock)) != NULL) {

		cdp->msg  = 0;
		cdp->mono = (disp_GetColors() < 16L) ? TRUE : FALSE;

		cdp->analog = TRUE;

		/*	we will draw with the colors used by the text attributes 
		 *	so, create a color map of the pixvals used in the attribute map.
		 */

		if (!cdp->mono) {
			if ((colors = ocolmap_Open(16)) == NULL) {
				cdp->mono = TRUE;
			}
			else {
				for (attr = 0x00; attr < 0x10; attr++) {
					pv = disp_GetAttrFgColor(attr);
					ocolmap_SetEntryPixval(colors, (int) attr, pv);
				}
			}
		}

		cdp->mono_clock.back   = CLOCK_MBACK;					
		cdp->mono_clock.hand   = CLOCK_MHAND;					
		cdp->mono_clock.sec	   = CLOCK_MSEC;					
		cdp->mono_clock.tick   = CLOCK_MTICK;					
		cdp->mono_clock.bord   = CLOCK_MBORD;  					
		
		cdp->color_clock.back  = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_CBACK);						
		cdp->color_clock.hand  = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_CHAND);					
		cdp->color_clock.sec   = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_CSEC);					
		cdp->color_clock.tick  = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_CTICK);					
		cdp->color_clock.bord  = cdp->mono ? CLOCK_MBORD : CLOCK_CBORD;					

		cdp->shadow_clock.back = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_SHADBACK);						
		cdp->shadow_clock.hand = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_SHADHAND);					
		cdp->shadow_clock.sec  = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_SHADSEC);					
		cdp->shadow_clock.tick = cdp->mono ? 0 : ocolmap_GetEntryPixval(colors, CLOCK_SHADTICK);					
		cdp->shadow_clock.bord = cdp->mono ? CLOCK_MBORD : CLOCK_SHADBORD;					

		if (colors != NULL) {
			ocolmap_Close(colors);
		}

		/*	initialize the tickmark coordinate array.

			a tickmark is a mark around the face of the clock denoting 
			a clock interval.  coordinates are stored in a tick_struct.
			the coordinates are the endpoints of a tickmark line.

			the face of the clock is circular.  we want to calculate the 
			tickmark positions in a manner that allows us to specify
			their length and position, relative to the size of the clock 
			circle.

			consider a circle, of radius r, whose center is at the origin 
			(0,0) and a point p, on the circle.  as the end point of a 
			vector from the origin, p is rotated through an angle theta 
			to point p' like this

				x'	= r * cos(theta) 
				y'	= r * sin(theta) 

			we calculate the endpoints tickmarks as points on 2 concentric
		 	circles, with the same origin as the clock circle, rotated through
			the same angle.  the distance between the points is the length
			of the tickmark.

			so that the tickmarks can be scaled with the clock, we conceive 
			of r as a percentage of the diameter of the clock circle.  r 
			may range from 0.0 to 100.0 (0% to 100%), inclusive.

			below, 12 tickmarks are calculated at regular intervals about the
			full circumference of the clock.  the length of a tickmark is 
			12% the diameter of the clock (40 - 28).  its distance from the
			center is 28% of the diameter of the clock; from the edge of the
			clock, 10% (100/2 - 40).

			(OAK_PI/2.0 is a quarter around the circle.)
		*/

		for (tickno = 0; tickno < 12; tickno++) {
			theta = (OAK_PI/2.0 - ((double)tickno * OAK_PI/6.0));
			cdp->tick[tickno].x1 = ((double)40.0) * (ourcos = cos(theta));
			cdp->tick[tickno].y1 = ((double)40.0) * (oursin = sin(theta));
			cdp->tick[tickno].x2 = ((double)28.0) * ourcos;
			cdp->tick[tickno].y2 = ((double)28.0) * oursin;
		}		
	}

	return;
}

/* -------------------------------------------------------------------------- */
int aux_Clock(win_type win, int msg, VOID *indata, VOID *outdata)
/*
	paints the clock, in shadow and in light.
*/
{
	ptd_struct 	   *ptd;
	opoint		   *p;
	clockdata_type	cdp;
	boolean			active;
	sed_type		sed;
	int				row, col, height, width, scancode;

	oak_notused(outdata);

	switch(msg) {

	case WINA_RESIZE:
		/*	don't let window be sized past display's right and bottom edge.
		 *	this code assumes the anchor is the upper-left corner of the
	 	 *	window--i.e., moused resizing by any border corner but the 
		 *	bottom-left will be crimped.
		 *
		 *	p->x is proposed width in pixels; p->y, height 
		 */
		p = (opoint *) indata;
		if ((bord_GetPixWidth(win) - win_GetPixWidth(win) + p->x) > (opcoord)disp_GetPixWidth() ||
	   		(bord_GetPixHeight(win) - win_GetPixHeight(win) + p->y) > (opcoord) disp_GetPixHeight()) {
			return(0);
		}
		break;
		
	case WINA_MOVE:
		/* don't let the window move off the display */
		demo_ClipMoveToDisplay(win, p = (opoint *) indata);
		break;

	case WINA_SHADOW:
		/*	fall through to WINA_PAINT ... */
	case WINA_PAINT:
		ptd = (ptd_struct *) indata;

		/*	let the draw routine know whether or not we are in shadow */
		((clockdata_type) win_GetData(ptd->win))->msg = msg;

		/* draw the clock */
		clock_Draw(ptd);

		break;

	case WINA_GO:
		active = TRUE;		
		while(active) {
			scancode = kb_Read();

			row 	= bord_GetTopRow(win);
			col 	= bord_GetLeftCol(win);
			height 	= win_GetHeight(win);
			width 	= win_GetWidth(win);

			switch(scancode) {

			case HELP_KEY:	/* key for non-DOS machines */
			case FN1:
				/*	this is general help for the clock */
				help_Show(24,0);
				break;
			
			case UP:
				bord_SetPosition(win, --row, col);
				break;
			case DOWN:
				bord_SetPosition(win, ++row, col);
				break;
			case LEFT:
				bord_SetPosition(win, row, --col);
				break;
			case RIGHT:
				bord_SetPosition(win, row, ++col); 
				break;

			case SHFT_UP:
				win_SetSize(win, --height < 1 ? 1 : height, width);
				break;
			case SHFT_DOWN:
				win_SetSize(win, ++height, width);
				break;
			case SHFT_LEFT:
				win_SetSize(win, height, --width < 1 ? 1 : width);
				break;
			case SHFT_RIGHT:
				win_SetSize(win, height, ++width);
				break;

			case TAB:
				if ((sed = obj_Find("ClockMsg")) != NULL ) {
					win_SetNextWin(win, sed);
					sed_Top(sed);
					active = FALSE;
				}
				break;

			case MOU_THERE:
			case ESC:
				active = FALSE;
				break;

			case MOU_NOFIELD:
			case MOU_CLICK:
				if (win == disp_GetTopWin()) {
					/*	if we are on top, switch from analog to digital and 
					 *	vice versa 
					*/
					cdp = (clockdata_type) win_GetData(win);
					cdp->analog = !(cdp->analog);
					win_Paint(win);
				}
				else {
					/* 	if we're not on top, top us */
					win_Top(win);
				}
				break;

			default:
				if (ascii(scancode) == ' ') {
					kb_Stuff(MOU_CLICK);
				}
				break;
			}
		
		}
		break;

	default:
		break;
	}

	return(1);
}

/* -------------------------------------------------------------------------- */
int aux_ClockMsg(sed_type sed, int msg, VOID *indata, VOID *outdata)
/*
	restricts movement on the clock message sed.
*/
{
	oak_notused(outdata);

	switch(msg) {

	case WINA_MOVE:
		/* don't let the window move off the display */
		demo_ClipMoveToDisplay(sed, (opoint *) indata);
		break;

	default:
		break;
	}

	return(1);
}

/* -------------------------------------------------------------------------- */
void clock_Draw(ptd_struct *ptd)
/*
	refresh the clock display
*/
{
	clockdata_type	cdp;
	double  		theta;
	int				back_color, tick_color, hand_color, sec_color;
	int				wid, hgt, hour, segwid;
	opoint 			pts[2];

	if (opbox_GetHeight(ptd->relboxp) <= 0
	|| opbox_GetWidth(ptd->relboxp) <= 0) {
	 	/*	nothing to draw */
		return;
	}

	/*	set up for the paint task */
	ogl_Start(ptd);
	ogl_SetLineWidth(2);

	cdp = (clockdata_type) win_GetData(ptd->win);
	cdp_GetColors(cdp, &back_color, &tick_color, &hand_color, &sec_color);

	ogl_SetColor(back_color);
	ogl_FillRect(ptd->relboxp);	

	if (!cdp->analog) {

		/* Digital clock: Draw 7 Segment Numerals */
		wid = win_GetPixWidth(ptd->win);
		hgt = win_GetPixHeight(ptd->win);
		hour = cdp->time.tm_hour % 12;
		hour = (hour == 0) ? 12 : hour;

		segwid = (3 * wid) / 20;

		if ((hour / 10) > 0) {
			/* draw leading 1, don't draw leading zero */
			seg7_DrawNum(ptd, 0, 8, segwid, hgt, hand_color, hour / 10);
		}
		seg7_DrawNum(ptd, ( 3 * wid) / 20, 8, segwid, hgt, hand_color, hour);
		seg7_DrawNum(ptd, ( 7 * wid) / 20, 8, segwid, hgt, hand_color, cdp->time.tm_min / 10);
		seg7_DrawNum(ptd, (10 * wid) / 20, 8, segwid, hgt, hand_color, cdp->time.tm_min);
		seg7_DrawNum(ptd, (14 * wid) / 20, 8, segwid, hgt, hand_color, cdp->time.tm_sec / 10);
		seg7_DrawNum(ptd, (17 * wid) / 20, 8, segwid, hgt, hand_color, cdp->time.tm_sec);

		/* Now put colons between digits */
		/* hour : minute */
		pts[0].x = pts[1].x = (6 * wid) / 20;
		pts[0].y = (3 * hgt) / 12;
		pts[1].y = (4 * hgt) / 12;

		ogl_SetColor(hand_color);
		ogl_DrawLine(pts);

		pts[0].y = (7 * hgt) / 12;
		pts[1].y = (8 * hgt) / 12;
		ogl_DrawLine(pts);

		/* minute : second */
		pts[0].x = pts[1].x = (13 * wid) / 20;

		pts[0].y = (3 * hgt) / 12;
		pts[1].y = (4 * hgt) / 12;
		ogl_DrawLine(pts);

		pts[0].y = (7 * hgt) / 12;
		pts[1].y = (8 * hgt) / 12;
		ogl_DrawLine(pts);

	}
	else {

		/* analog clock */
	
		/*	12 tickmarks */
		clock_DrawTicks(ptd, tick_color);
	
		/* 	second hand, its length is 40% of clock diameter */
		theta = (OAK_PI/2.0) - (cdp->time.tm_sec * (OAK_PI/30.0));
		clock_DrawHand(ptd, theta, (double)40.0, TRUE, sec_color);
	
		/*	minute hand, its length is 30% of clock diameter */
		theta = (OAK_PI/2.0) - (cdp->time.tm_min * (OAK_PI/30.0)) ;
		clock_DrawHand(ptd, theta, (double)30.0, FALSE, hand_color);
	
		/* 	hour hand,   its length is 20% of clock diameter */
		theta = (OAK_PI/2.0) - (((cdp->time.tm_hour) * 60.0 + cdp->time.tm_min)
					* OAK_PI) / 360.0;
		clock_DrawHand(ptd, theta, (double)20.0, FALSE, hand_color);
	}
	
	ogl_Stop();
 	return;
}

/* -------------------------------------------------------------------------- */
void clock_DrawTicks(ptd_struct *ptd, int color)
/*
	Draws the tick marks around the clock.

	This routine is called by clock_Draw, so does not 
	use ogl_Start/ogl_Stop.
*/
{
	opbox			dimbox;
	opcoord			wid, hgt;
	clockdata_type 	cdp;
	int				tickno;
	opoint 			pts[2];

	cdp = (clockdata_type) win_GetData(ptd->win);

	/*	for TURBO C--the line drawing routines are clip region relative */

	win_GetPixBox(ptd->win, &dimbox);
	wid = opbox_GetWidth(&dimbox);
	hgt = opbox_GetHeight(&dimbox);

	/* 	draw the 12 tick marks on the clock face from the 
		pre-calculated array of relative positions
	*/
	for (tickno = 0; tickno < 12; tickno++) {

		/* scale and translate draw coords to window size */
		pts[1].x = dimbox.xmin + 
			((int) (((long) ((double)50.0 + cdp->tick[tickno].x2) * (long) wid) / 100L));
		pts[1].y = dimbox.ymin + 
			((int) (((long) ((double)50.0 - cdp->tick[tickno].y2) * (long) hgt) / 100L));
		pts[0].x = dimbox.xmin + 
			((int) (((long) ((double)50.0 + cdp->tick[tickno].x1) * (long) wid) / 100L));
		pts[0].y = dimbox.ymin + 
			((int) (((long) ((double)50.0 - cdp->tick[tickno].y1) * (long) hgt) / 100L));

		ogl_SetColor(color);
		ogl_DrawLine(pts);
	}
	return;
}

/* -------------------------------------------------------------------------- */
void clock_DrawHand(ptd_struct *ptd, double theta, double radius, boolean sec, int color)
/*
	theta is the radian measure of the angle of the hand's axis.

	radius is the radius of the circle used to calculate the endpoints of the
	hand.  it represents a percentage of the extent of the window.

	sec is a flag denoting whether to the hand is the second hand.  if so, 
	a single line is drawn; else a triangular outline of the hour or
	minute hand is drawn.

	This routine is called by clock_Draw, so does not use ogl_Start/ogl_Stop.
*/
{
	double	tipx, tipy;
	opbox	dimbox;
	opcoord	wid, hgt;
	int		xcenter, ycenter;
	opoint 	pts[2];

	win_GetPixBox(ptd->win, &dimbox);
	wid = opbox_GetWidth(&dimbox);
	hgt = opbox_GetHeight(&dimbox);

	/*	the outermost tip of the hand */
	tipx = radius * cos(theta);
	tipy = radius * sin(theta);

	/*	translate (relative to clock center, 50% of clock dimension) 
		and scale 
	*/
	pts[1].x = dimbox.xmin + 
		(int) (((((double) 50.0) + tipx) * (double)wid) * (double)0.01);
	pts[1].y = dimbox.ymin + 
		(int) (((((double)50.0) - tipy) * (double)hgt) * (double)0.01);
	pts[0].x = dimbox.xmin + 
		(int) ((((double)50.0) * (double)wid) * (double)0.01);
	pts[0].y = dimbox.ymin + 
		(int) ((((double)50.0) * (double)hgt) * (double)0.01);

	xcenter = pts[0].x;
	ycenter = pts[0].y;

	ogl_SetColor(color);

	if (sec) {

		/*	draw the second hand from the tip we set up on the periphery of
			the clock to the clock center
		*/
		ogl_DrawLine(pts);
	}
	else {
		/*	hour or minute hand rendering */

		double 	basex1, basey1, basex2, basey2;
		int 	tempx, tempy;

		/*	figure an angle (almost, OAK_PI/1.6) perpendicular to the hand axis.
			form a short vector which, together with the tip point calculated
			above,  denotes a line that will render one side of the hand.
			the endpoint calculated here is at the broad end
			of the hand near the center of the clock.
		*/
		basex1 = ((double)2.0) * cos(theta - OAK_PI/2.0);
		basey1 = ((double)2.0) * sin(theta - OAK_PI/2.0);
		basex2 = ((double)2.0) * cos(theta + OAK_PI/2.0);
		basey2 = ((double)2.0) * sin(theta + OAK_PI/2.0);

		/*	(drawbox max has the tip) put base pt in drawbox min to 
			draw one side of the hand 
		*/
		pts[0].x = dimbox.xmin + 
			(int) (((((double)50.0) + basex1) * (double)wid) * (double)0.01);
		pts[0].y = dimbox.ymin + 
			(int) (((((double)50.0) - basey1) * (double)hgt) * (double)0.01);
		ogl_DrawLine(pts);

		/*	save this base endpoint */
		tempx = pts[0].x;
		tempy = pts[0].y;

		/*	put in the other base endpoint--plot to tip */
		pts[0].x = dimbox.xmin + 
			(int) (((((double)50.0) + basex2) * (double)wid) * (double)0.01);
		pts[0].y = dimbox.ymin + 
			(int) (((((double)50.0) - basey2) * (double)hgt) * (double)0.01);
		ogl_DrawLine(pts);

		/* 	plot from endpoints to center */
		pts[1].x = xcenter;
		pts[1].y = ycenter;
		ogl_DrawLine(pts);

		pts[0].x = tempx;
		pts[0].y = tempy;
		ogl_DrawLine(pts);
	}

 	return;
}

/* 7 Segment display routines */

/*
 *	This is a table of which segments should be turned on for 
 *	each number.  Each bit corresponds to one of the 7 segments.  The 
 *	segments are mapped as follows:
 *
 *	  1			 0 is the LSB.
 *	0	2
 *	  3
 *	4	6
 *	  5
 *
 */

static byte seg7_table[] = {0x77,0x44,0x3e,0x6e,0x4d,0x6b,0x7b,0x46,0x7f,0x4f};

void seg7_DrawNum(ptd_struct *ptd, int x, int y, int wid, int hgt, opixval fg, int num)
/*
	Plot a number, num, as a 7 segment "LED" numeral.
	x,y is the location within the window to plot the numeral.
	wid and hgt are the height and width of the numeral (in pixels)
	fg is the color of the numeral. (assumes the bacground has been cleared)

	This routine is called by clock_Draw, so does not use ogl_Start/ogl_Stop.
*/
{
	opoint	pts[2];

	oak_notused(ptd);

	/* Save some space around the edges */
	wid -= 2 + wid/8;
	hgt -= 2 + hgt/8;
	wid = (wid < 2) ? 2 : wid;
	hgt = (hgt < 2) ? 2 : hgt;

	/* Draw the segments */

	/* Make sure num is within the right range */
	num %= 10;

	ogl_SetColor(fg);

	/* segment 0 */
	if (seg7_table[num]	& 0x01) {
	
		pts[0].x = x;
		pts[0].y = y + 1;
		pts[1].x = x;
		pts[1].y = y + hgt/2 - 1;
		ogl_DrawLine(pts);
	}

	/* segment 1 */
	if (seg7_table[num]	& 0x02) {
	
		pts[0].x = x + 1;
		pts[0].y = y;
		pts[1].x = x + wid - 1;
		pts[1].y = y;
		ogl_DrawLine(pts);
	}

	/* segment 2 */
	if (seg7_table[num]	& 0x04) {
	
		pts[0].x = x + wid;
		pts[0].y = y + 1;
		pts[1].x = x + wid;
		pts[1].y = y + hgt/2 - 1;
		ogl_DrawLine(pts);
	}

	/* segment 3 */
	if (seg7_table[num]	& 0x08) {
	
		pts[0].x = x + 1;
		pts[0].y = y + hgt/2;
		pts[1].x = x + wid - 1;
		pts[1].y = y + hgt/2;
		ogl_DrawLine(pts);
	}

	/* segment 4 */
	if (seg7_table[num]	& 0x10) {
	
		pts[0].x = x;
		pts[0].y = y + hgt/2 + 1;
		pts[1].x = x;
		pts[1].y = y + hgt - 1;
		ogl_DrawLine(pts);
	}

	/* segment 5 */
	if (seg7_table[num]	& 0x20) {
	
		pts[0].x = x + 1;
		pts[0].y = y + hgt;
		pts[1].x = x + wid - 1;
		pts[1].y = y + hgt;
		ogl_DrawLine(pts);
	}

	/* segment 6 */
	if (seg7_table[num]	& 0x40) {
	
		pts[0].x = x + wid;
		pts[0].y = y + hgt/2 + 1;
		pts[1].x = x + wid;
		pts[1].y = y + hgt - 1;
		ogl_DrawLine(pts);
	}
}

/* -------------------------------------------------------------------------- */
void cdp_GetColors(clockdata_type cdp, int *back_color, int *tick_color, int *hand_color, int *sec_color)
{
	if (!cdp->mono) {
		if (cdp->msg == WINA_SHADOW) {
			*back_color = cdp->shadow_clock.back;
			*tick_color = cdp->shadow_clock.tick;
			*hand_color = cdp->shadow_clock.hand;
			*sec_color  = cdp->shadow_clock.sec;
		}
		else {
			*back_color = cdp->color_clock.back;
			*tick_color = cdp->color_clock.tick;
			*hand_color = cdp->color_clock.hand;
			*sec_color  = cdp->color_clock.sec;
		}
	}
	else {
		*back_color = cdp->mono_clock.back;
		*tick_color = cdp->mono_clock.tick;
		*hand_color = cdp->mono_clock.hand;
		*sec_color  = cdp->mono_clock.sec;

		if (cdp->msg == WINA_SHADOW) {
			*back_color = *back_color ? 0 : 1;
			*tick_color = *tick_color ? 0 : 1;
			*hand_color = *hand_color ? 0 : 1;
			*sec_color  = *sec_color  ? 0 : 1;
		}
	}
	return;
}

/* -------------------------------------------------------------------------- */
boolean spc_ClockMsg(sed_type sed, int scancode) 
/*
	special function for the clock message box.
	
	handles keystrokes for clock window to move, resize, toggle 
	analog/digital face.
*/								
{
	win_type 		clock;
	clockdata_type	cdp;
	int				row, col, height, width;
	
	
	if ((clock = obj_Find("Clock")) == NULL) {
		/*	do nothing if no clock not found */
	 	return(FALSE);
	}

	row 	= bord_GetTopRow(clock);
	col 	= bord_GetLeftCol(clock);
	height 	= win_GetHeight(clock);
	width 	= win_GetWidth(clock);

	switch(scancode) {

	case MOU_NOFIELD:
		/*	top us */
		win_Top(sed);
		break;
		
	case TAB:
		win_Top(clock);
		sed_SetNextWin(sed, clock);
		sed_ToggleExit(sed);
		break;		

	case UP:
		bord_SetPosition(clock, --row, col);
		break;
	case DOWN:
		bord_SetPosition(clock, ++row, col);
		break;
	case LEFT:
		bord_SetPosition(clock, row, --col);
		break;
	case RIGHT:
		bord_SetPosition(clock, row, ++col); 
		break;
	
	case SHFT_UP:
		win_SetSize(clock, --height < 1 ? 1 : height, width);
		break;
	case SHFT_DOWN:
		win_SetSize(clock, ++height, width);
		break;
	case SHFT_LEFT:
		win_SetSize(clock, height, --width < 1 ? 1 : width);
		break;
	case SHFT_RIGHT:
		win_SetSize(clock, height, ++width) ;
		break;
	
	default:
		if (ascii(scancode) != ' ') {
	 		return(FALSE);
		}
		else {
			/*	toggle analog/digital face when spacebar pressed */
			cdp = (clockdata_type) win_GetData(clock);
			cdp->analog = !(cdp->analog);
			win_Paint(clock);
		}
	}

	return(TRUE);
}


