/*
	csdemo.c

	jmd 9/90

	% main module of the C-scape Demo.

	C-scape 4.0.1 Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	 1/09/91 pmcm	loaded menubar from sfile
	 1/09/91 pmcm	replace AUX_START msg with WINA_GO in aux_FileList
	 1/10/91 pmcm	removed bd_std, _plain, _box, _2; added bd_mouse2
	 1/10/91 pmcm	added aux_Data1 for data initialization
	 1/10/91 pmcm	added aux_Borders
	 1/12/91 pmcm	added demo_Text
	 1/12/91 pmcm	removed card demo stuff
	 1/12/91 pmcm	preened fsymlist
	 1/17/91 pmcm	added help/escape buttons and demo_ClipMove
	 1/17/91 pmcm	added DEMO_TEXTONLY, DEMO_GRAFONLY support
	 1/19/91 bkd	changed return(0) in main to exit(0); return(0).
	 1/22/91 pmcm	added clock
	 1/27/91 pmcm	added help_Close, oak_Close
	 1/27/91 pmcm	added demo_Close
	 1/28/91 pmcm	added dgraf_Init to graphics initialization
	 1/28/91 pmcm	added init and close msgs for when mode switching
	 2/04/91 ted	tweaked clear color at close time so DOS screen is normal.
	 2/06/91 jmd/jdc	added monochrome flag to command line, removed exp_Bmu
	 2/07/91 pmcm	added usermono, mono flags, demo_SetMono
	  4/2/91 bkd	added X support, added pmap_IoInit after disp_Init
	  4/3/91 bkd	added "void" to empty argument lists
	  7/3/91 bkd	updated for new X support.
	 7/15/91 bkd	restructured so that DEMO_TEXTONLY will not even link
	 				in demo modules with pmap code.
	 7/23/91 bkd	updated to C-scape 3.2c demonstration program.

	12/19/91 pmcm	converted to use OGL.
	 1/02/92 pmcm	bd_cua'd
	 1/13/92 pmcm	revised to allow text-and-graphics compilation

	 2/26/92 pmcm	updated osys_InitSettings() calls
	 2/28/92 pmcm	fixed graf to textmode switch,first time into textmode case.
	 3/04/92 pmcm	replaced _ClassInit with _MouseInit	
	 5/01/92 pmcm	changed all module names from cs32... to cs...;
					added bdcua_Init calls
	 6/03/92 jdc	added error checks and disp_Close on error
	 6/08/92 pmcm	removed demo of standard borders.
	 6/15/92 pmcm	added out-of-memory routine and error checks; added
					aux_NoControl() for clicks on field-less sed CUA borders.
	 7/31/92 pmcm 	turned off mouse under Herc graphics with OGL
	 7/31/92 pmcm	diddled close so we exit in graphics mode if dual mode.
	 8/05/92 pmcm	removed TEXTONLY, GRAFONLY
*/

#include "csdemo.h"			/*	contains common C lib and C-scape headers */

#include "framer.h"
#include "helpdecl.h"
#include "popdecl.h"
#include "teddecl.h"
#include "cstime.h"
#include "oaktag.h"
#include "oakalloc.h"
#include "dpref.h"
#include "ogldecl.h"


/***------------------------ Local Function Declaration Prototypes -------***/

void 			main_loop(void);

/***------------------------- Demo Globals  ------------------------------***/

/* global data used throughout the demo */
OGLOBAL struct demodata_struct demodata = {
	NULL,	 	/* the demo screen file */
	NULL,		/* the current 	main screens */
	NULL,	 	/* the text 	mode screens */
	NULL,	 	/* the graphics mode screens */
	NULL,		/* the current help sed for the help button */
	NULL,	 	/* the text 	mode Device Interface */
	NULL,		/* the graphics mode Device Interface */
	TRUE,		/* can we do graphics? */
	FALSE,		/* are we in text mode? */
	FALSE,		/* user's command line mono flag */
	FALSE,		/* hardware monochrome flag */
	MODE_MENU,	/* start out in menu mode */
	0,		  	/* graflib uninitialized */
	NULL		/* memory stash for out-of-memory popup */
};

OGLOBAL char *SFILEOPENFAIL_MSG 	= "CSDEMO: unable to open the csdemo.lnf file.\n";

/***------------------------- Module Globals -----------------------------***/

/*	messages 
 *	(all other messages are stored in MsgText screen of the csdemo.lnf file)
 */

char *MEMCACHEFAIL_MSG 		= "CSDEMO: unable to allocate demo memory cache.\n";
char *INITFAIL_MSG 			= "CSDEMO: unable to initialize device.\n";
char *SFILELOADFAIL_MSG 	= "CSDEMO: insufficient memory to load screens.\n";

/* ------------------------------------------------------------------------ */

int main(int argc, char *argv[])
{
	boolean		 ok = FALSE;

	/*	platform-specific pre-initialization display preferences (dp_) */

	dp_SetApplName("xcsdemo");
	dp_SetWindowName("C-scape 4.0.1 Demonstration program");
	argc = dp_PickArgs(argc, argv);

	/* 	the "-m" or "-M" command line argument requests monochrome mode */

	if (argc > 1 && argv[1][0] == '-' && otolower(argv[1][1]) == 'm') {
		demodata.usermono = TRUE;
	}

	/*	initialize the device interface and open the principal screens */

	switch (demo_Open()) {

	case DEMO_MEMCACHEFAIL:
		printf("%s", MEMCACHEFAIL_MSG ); 	
		break;
	case DEMO_INITFAIL:		
		printf("%s", INITFAIL_MSG ); 	
		break;
	case DEMO_SFILEOPENFAIL:	
		printf("%s", SFILEOPENFAIL_MSG); 	
		break;
	case DEMO_SFILELOADFAIL:	
		printf("%s", SFILELOADFAIL_MSG); 	
		break;
	case DEMO_NOFAIL:	
	default:					
		ok = TRUE;							
		break;
	}

	if (!ok) {
		exit(1);
		return(1);
	}

	main_loop();

	demo_Close();

	exit(0);
	return(0);
}
/* ------------------------------------------------------------------------ */

scrn_struct *demo_OpenScrns(void)
/*
	This routine creates the background sed, the prompt and main menubar.

	It returns a pointer to the background sed.
	The background sed contains a pointer to the menubar.
*/
{
	scrn_struct    *scrns;
	sed_type		menubar;
	sed_type		promptbar;
	sed_type		back;
	sed_type		button;
	menu_type 		menu;
	register int 	i;

	if ((scrns = (scrn_struct *) omalloc(OA_NOTAG, sizeof(scrn_struct))) == NULL) {
	 	return(NULL);
	}
	
	/* create background screen */
	if ((menu = menu_Open()) == NULL) {
		goto QUIT1;
	}

	/* 	fill up the background screen (alas, octal is most portable) */
	for (i = 0; i < disp_GetHeight() - 1; i++) {
	    if (!menu_Printf(menu, "@[%d,\260]\n", disp_GetWidth())) {
			menu_Destroy(menu);
			goto QUIT1;
		}
	}
	
	menu_Flush(menu);

	if ((back = sed_Open(menu)) == NULL) {
		menu_Destroy(menu);
		goto QUIT1;
	}
	sed_SetColors(back, 0x78, 0x78, 0x78);
	sed_SetShadowAttr(back, 0x08);

	/* MENUBAR:
	 *
	 */
	if ((menubar = demo_LoadSed("MenuBar", 0)) == NULL) {
		goto QUIT2;
	}
	sed_SetWidth(menubar, disp_GetWidth());

#ifdef OAK_DOS
	/*	This diddling is to ensure we get the attributes we want in the
	 *	menubar when running on DOS machines with monochrome CGA
	 *	displays (like the Compaq Plasma display) or monochrome
	 *	Hercules displays.
	 */ 
	{
	sed_type bob;

	if (demodata.mono) {
		if (demo_IsTextMode()) {
			/*	text mode monochrome, change the colors
			 *	in the top menubar
			 */

			sed_SetColors(menubar, 0x01, 0x01, 0x10);
			sed_SetHiColors(menubar, 0x0f, 0x10);						

			/*	and the colors in the pulldowns */

	        for(i = sed_GetFieldCount(menubar); i > 0; i--) {
	            if (((bob = sed_GetFieldBob(menubar, i - 1)) != NULL) &&
	              	bob_IsSed(bob)) {

					sed_SetColors(bob, 0x01, 0x01, 0x10);
					sed_SetHiColors(bob, 0x0f, 0x10);						
		        }
			}
		}
		else {
			/*	graphics mode monochrome, change the hichar colors 
			 *	in the top menubar only
			 */
			sed_SetHiColors(menubar, 0x10, 0x01);						
		}		
	}
	}
#endif


	/*	PROMPTBAR:
	 *
	 */
	if ((promptbar = demo_LoadSed("PromptBar", 0)) == NULL) {
		goto QUIT3;
	}
	sed_SetPosition(promptbar, disp_GetHeight() - 1, 0);
	sed_SetWidth(promptbar, disp_GetWidth());

	/*	BUTTONS:
	 *	(the escape and help buttons that appear over the promptbar)
	 */
	if ((button = demo_LoadSed("EscapeButton", 0)) == NULL) {
		goto QUIT4;
	}
	sed_SetPosition(button, disp_GetHeight() - 1, 
			disp_GetWidth() - bord_GetWidth(button));
	scrns->buttons[ESCBTN] = button;

	if ((button = demo_LoadSed("HelpButton", 0)) == NULL) {
		goto QUIT5;
	}
	sed_SetPosition(button, disp_GetHeight() - 1, 
			disp_GetWidth() - bord_GetWidth(button) - bord_GetWidth(scrns->buttons[ESCBTN]));
	scrns->buttons[HELPBTN] = button;


	/*	TEXT:
	 *	load up the text of the various messages we will use.
	 *	we store these in an screen file to facilitate translation.
	 */
	if ((scrns->text = demo_LoadSed("MsgText", 0)) == NULL) {
		goto QUIT6;
	}
	/* 	paint the backdrop first */
	sed_Repaint(scrns->back			= back);
	sed_Repaint(scrns->menubar 		= menubar);
	sed_Repaint(scrns->promptbar 	= promptbar);

	return(scrns);

QUIT6:
	sed_Close(scrns->buttons[HELPBTN]);
QUIT5:
	sed_Close(scrns->buttons[ESCBTN]);
QUIT4:
	sed_Close(promptbar);
QUIT3:
	sed_Close(menubar);
QUIT2:
	sed_Close(back);
QUIT1:
	ofree(OA_NOTAG, (VOID *)scrns);

	return(NULL);
}

/* ------------------------------------------------------------------------ */
void demo_CloseScrns(void)
/*
	Close current mode (text or graphics) demo screens.
*/
{
#ifdef DEMO_CLEANCLOSE
	if (demo_GetCurrScrns() != NULL) {
		if (demo_GetText() != NULL) {
			sed_Close(demo_GetText());
		}
		if (demo_GetBack() != NULL) {
			sed_Close(demo_GetBack());
		}
		if (demo_GetPromptBar() != NULL) {
			sed_Close(demo_GetPromptBar());
		}
		if (demo_GetMenuBar() != NULL) {
			sed_Close(demo_GetMenuBar());
		}
		if (demo_GetButton(ESCBTN) != NULL) {
			win_Close(demo_GetButton(ESCBTN));
		}
		if (demo_GetButton(HELPBTN) != NULL) {
			win_Close(demo_GetButton(HELPBTN));
		}

		/*	free up the current screens structure */
		ofree(OA_NOTAG, demo_GetCurrScrns());
		demo_SetCurrScrns(NULL);

		demo_IsTextMode() ? demo_SetTextScrns(NULL) : demo_SetGrafScrns(NULL);
	}
#endif
 	return;
}
/* ------------------------------------------------------------------------ */

void demo_SetMode(int mode)
/*
	Change the operating mode of the demo.

	mode is one of the following:
		MODE_MENU
		MODE_DEMO
*/
{
	int 	i;
	byte	reg, bck, sel;

	switch(mode) {
	case MODE_DEMO:
		/* 	yank away the mouse handler from the menubar so control does not
		 * 	pass to it.  un-highlight the hi-chars and mark the fields
		 *	to show the menubar is inactive
		 */
		for (i = 0; i < sed_GetFieldCount(demo_GetMenuBar()); i++) {
				/*	the -1 parameter diables the hichar visibility for the
				 * 	specified field.  the position of the hichar remains
				 *	intact.
				 */
				sed_SetFieldHiChar(demo_GetMenuBar(), i, -1);
				if (!sed_IsMarkedField(demo_GetMenuBar(), i)) {
					sed_MarkField(demo_GetMenuBar(), i, 0x40, 0x40);
				}
		}
		sed_GetColors(demo_GetMenuBar(), &reg, &bck, &sel);
		sed_SetColors(demo_GetMenuBar(), reg, 0x40, sel);
		sed_SetMouse(demo_GetMenuBar(), FNULL);
		sed_Update(demo_GetMenuBar());
		break;

	case MODE_MENU:
		/* 	hook up the menubar mouse handler and re-highlight the hi-chars */
		for (i = 0; i < sed_GetFieldCount(demo_GetMenuBar()); i++) {
			/*	use the current hichar position.  if it is between 0-15,
			 *	then hichar visibility is (re-)enabled.
			 */
			sed_SetFieldHiChar(demo_GetMenuBar(), i, menu_GetHiChar(sed_GetMenu(demo_GetMenuBar()), i));
			if (sed_IsMarkedField(demo_GetMenuBar(), i)) {
				sed_UnMarkField(demo_GetMenuBar(), i);
			}
		}
		sed_GetColors(demo_GetMenuBar(), &reg, &bck, &sel);
		sed_SetColors(demo_GetMenuBar(), reg, reg, sel);
		sed_SetMouse(demo_GetMenuBar(), sedmou_Framer);
		sed_Update(demo_GetMenuBar());

		/* hide buttons in case they are left on */
		demo_HideButton(ESCBTN);
		demo_HideButton(HELPBTN);
		demo_SetButtonData(HELPBTN,NULL);

		break;

	default:
		break;
	}

	demodata.opmode = mode;
}

/* ------------------------------------------------------------------------ */
void main_loop(void)
/*
	This is the central part of the program.
	Here we sit in a loop processing events	until we're finished.
*/
{
	boolean quit = FALSE;

	demo_SetMode(MODE_MENU);

	/* 	here is the main event loop.

		control is passed to the pulldown menu-ing system at the top of
		the display.  the use makes a choice from the menu and causes a
		pre-defined command code value to be returned to the switch, below.  
		each case in the switch carries out a particular command by running
		a portion of the demo. 

		MODE_MENU enables the pulldown menuing system.  MODE_DEMO disables
		the pulldown menuing system while a command is carried out.

		the loop continues until the Escape key is pressed (switch on 0)
		from the menuing system, or the Exit command is issued.
	*/

	/* the main loop... */
	while (!quit) {

		switch (frame_Go(demo_GetMenuBar(), (char) ' ', NULL)) {

		case 0:	   /* Escape pressed from menubar  */
		case DEMO_QUIT:
			/* ask if it's ok to quit */
			quit = (demo_Query(demo_Text("quit_prompt")) == 1);
			break;

		case DEMO_GRAPHICS:
			/*	put demo into graphics mode */
			if (demo_IsTextMode()) {
				if (!demo_IsGraphicsCapable()) {
					demo_LoadSed("NoGraphics", DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
				}
				else if (!demo_SwitchToGraphicsMode()) {
					demo_LoadSed("NoGraphics", DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
				}
			}
			break;

		case DEMO_TEXT:
			/* 	put demo into text mode */
			if (!demo_IsTextMode()) {
#ifdef DEMO_NOMODESWITCH
				demo_LoadSed("NoText", DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
#else
				if (!demo_SwitchToTextMode()) {
					demo_LoadSed("NoText", DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
				}
#endif	
			}
			break;

		case DEMO_HYPERFUN:
			/* 	start the tutorial: call the help system */
			oak_ClearAllocErrorTag();
			help_Show(1, 1);
			if (oak_GetAllocErrorTag() != 0) {
				oak_ClearAllocErrorTag();
				demo_MemOut(demo_Text("helpmemout"));
			}
			break;

		case DEMO_GENERAL:
			/* 	call the introductory screen for the help system */
			oak_ClearAllocErrorTag();
			help_Show(6, 1);
			if (oak_GetAllocErrorTag() != 0) {
				oak_ClearAllocErrorTag();
				demo_MemOut(demo_Text("helpmemout"));
			}
			break;

		case DEMO_HELP:
			/* same as pressing F1 */
			kb_Stuff(FN1);
			break;

		case DEMO_FUNCS:
			/* view list of C-scape functions */
			demo_SetMode(MODE_DEMO);
			demo_ShowButton(ESCBTN);
			demo_ShowButton(HELPBTN);

			/* 	(help button data is set by demo_ViewFile) */
			demo_ViewFile("csfunc.txt");

			demo_HideButton(ESCBTN);
			demo_HideButton(HELPBTN);
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_SOURCE:
			/* view a demo source file */
			{
				sed_type filelist, filemsg;
				int		 choice;

				demo_SetMode(MODE_DEMO);

				/* 	load FileMsg screen first so aux_FileList can
				 *	update its direction field on startup.
				 */
				if ((filemsg = demo_LoadSed("FileMsg", DL_PAINT)) == NULL
				|| (filelist = demo_LoadSed("FileList", DL_PAINT)) == NULL) {

					if (filemsg != NULL) {
						sed_Close(filemsg);
					}
					demo_MemOut(demo_Text("memout"));
					demo_SetMode(MODE_MENU);
					break;
				}

				demo_ShowButton(ESCBTN);
				demo_ShowButton(HELPBTN);

				while (1) {
					/*	set this everytime around because ViewFile messes with
					 *	it, too
					 */
					demo_SetButtonData(HELPBTN, (VOID *) filelist);
	
					if ((choice = sed_Go(filelist)) == 0) {
						break;
					}
					else {
						demo_ViewFile(sed_GetMerge(filelist, choice - 1));
					}
				}

				demo_HideButton(ESCBTN);
				demo_SetButtonData(HELPBTN, NULL);
				demo_HideButton(HELPBTN);

				sed_Close(filemsg);
				sed_Close(filelist);
				demo_SetMode(MODE_MENU);
			}
			break;

		case DEMO_ABOUT:
			demo_SetMode(MODE_DEMO);
			demo_LoadSed("About", DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_SYSTEMS:
			demo_SetMode(MODE_DEMO);
			demo_LoadSed("Systems", DL_PAINT | DL_CENTER | DL_GO | DL_CLOSE | DL_MEMOUT);
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_WINDOWS:
			demo_SetMode(MODE_DEMO);
			if (!demo_Windows()) {
			 	demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_BORDERS:
			demo_SetMode(MODE_DEMO);
			/*	border demo does own memout check */
			demo_Borders();
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_DATA1:
			demo_SetMode(MODE_DEMO);
			demo_LoadSed("Data1", DL_PAINT | DL_CENTER | DL_GO | DL_CLOSE | DL_ESCBTN | DL_HELPBTN | DL_MEMOUT);
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_STOCK:
			demo_SetMode(MODE_DEMO);
			/*	stock demo does own memout check */
			demo_Stock();
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_POPUP:
			demo_SetMode(MODE_DEMO);
			if (!demo_Popup()) {
				demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_PULLDOWN:
			demo_SetMode(MODE_DEMO);
			if (!demo_Pulldown()) {
				demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_123:
			demo_SetMode(MODE_DEMO);
			if (!demo_Slug(MENU_HORIZONTAL)) {
				demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_SLUG:
			demo_SetMode(MODE_DEMO);
			if (!demo_Slug(MENU_VERTICAL)) {
				demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_WORLD:
			/* call the world demo */
			demo_SetMode(MODE_DEMO);
			if (!demo_World()) {
			 	demo_MemOut(demo_Text("memout"));
			}
			demo_SetMode(MODE_MENU);
			break;

		case DEMO_TICTAC:
			if (!demo_IsGraphicsCapable()) {
				demo_LoadSed("NoGraphics",
					DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
			}	
			else {
				if (demo_IsTextMode()) {
					demo_LoadSed("NeedsGraphics",
						DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
				}
				else {
					demo_SetMode(MODE_DEMO);
					if (!demo_TicTac()) {
					 	demo_MemOut(demo_Text("memout"));
					}
					demo_SetMode(MODE_MENU);
				}
			}
			break;

		case DEMO_CLOCK:
			if (!demo_IsGraphicsCapable()) {
				demo_LoadSed("NoGraphics",
					DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
			}
			else {
				if (demo_IsTextMode()) {
					demo_LoadSed("NeedsGraphics",
						DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
				}
				else {
					if (ogl_Ok()) {
						demo_SetMode(MODE_DEMO);
						if (!demo_Clock()) {
							demo_MemOut(demo_Text("memout"));
						}
						demo_SetMode(MODE_MENU);
					}
					else {
						demo_LoadSed("NeedsGraphicsLib",
					  		DL_CENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
					}
				}
			}
			break;

		default:
			break;
		}
	}
}

/* ------------------------------------------------------------------------ */
sed_type demo_LoadSed(char *name, unsigned mode)
/*
 *	Load the screen with the given name from the demo sfile.
 *
 *	Depending on mode, paint, go, and/or close the screen.
 *
 *	If not closed, return the sed.
 *
 *	mode is a combination of DL_PAINT, DL_GO, DL_CLOSE, DL_CENTER, DL_HCENTER,
 *	DL_SIZE, DL_ESCBTN, DL_HELPBTN, or DL_MEMOUT
 */
{
	sed_type 	sed;
	int			row, col;

	if ((sed = sfile_LoadSed(demodata.demo_sfile, name, SED_ALLOC)) != NULL) {

		if (mode & DL_CENTER) {
			/* center the sed on the display 
			 * (we might be running on an odd sized display)
			 */
			sed_Center(sed);
		}
		if (mode & DL_HCENTER) {
			/* center the sed horizontally on the display */
			sed_GetPosition(sed, &row, &col);
			sed_SetPosition(sed, row, (disp_GetWidth() - bord_GetWidth(sed))/2);
		}

		if (mode & DL_SIZE) {
			/*	fill the display, but leave the prompt bar showing */
			sed_SetWidth(sed, sed_GetWidth(sed) + disp_GetWidth() - bord_GetWidth(sed));
			sed_SetHeight(sed, sed_GetHeight(sed) + disp_GetHeight() - bord_GetHeight(sed) - 1);
			bord_SetPosition(sed, 0, 0);
		}

		if (mode & DL_PAINT) {
			sed_Repaint(sed);
		}

		if (mode & DL_ESCBTN) {
			demo_ShowButton(ESCBTN);
		}

		if (mode & DL_HELPBTN) {
			demo_SetButtonData(HELPBTN, (VOID *) sed);
			demo_ShowButton(HELPBTN);
		}
		
		if (mode & DL_GO) {
			sed_Go(sed);

			if (mode & DL_ESCBTN) {
				demo_HideButton(ESCBTN);
			}
			if (mode & DL_HELPBTN) {
				demo_HideButton(HELPBTN);
				demo_SetButtonData(HELPBTN, NULL);
			}
		}

		if (mode & DL_CLOSE) {
			sed_Close(sed);
			sed = NULL;
		}
	}
	else {
	 	/*	unable to load sed */
		if (mode & DL_MEMOUT) {
	   		demo_MemOut(demo_Text("memout"));
		}
	}

	return(sed);
}

/* ------------------------------------------------------------------------ */
int demo_Query(char *msg)
/*
	Puts up a YES/NO question box.
	Returns 0 if ESC pressed,
			1 for Yes
			2 for No
*/
{
	sed_type query_sed;
	int    	 ret = 0;

	/* load the screen */
	if ((query_sed = demo_LoadSed("Query", DL_CENTER)) != NULL) {

		/* Put the msg in the border title */
		sed_SetBorderTitle(query_sed, msg);

		sed_Repaint(query_sed);
		ret = sed_Go(query_sed);
		sed_Close(query_sed);
	}
	else {
		/*	quit if we cannot load the query */
		ret = 1;
	}
	return(ret);
}

/* ------------------------------------------------------------------------ */
boolean demo_ViewFile(char *filename)
/*
	Open and view the file 'filename'.
*/
{
	sed_type 	ted;
	FILE 	   *fp;

	if ((fp = fopen(filename, "r")) == NULL) {
		demo_FileMissing(filename);
	}
	else {
		if ((ted = demo_LoadSed("Editor", DL_SIZE | DL_MEMOUT)) != NULL) {
			sed_SetBorderTitle(ted, filename);
			if (ted_ReadFile(ted, fp) == -1) {
			 	demo_MemOut(demo_Text("viewfilefail"));
			}
			sed_Repaint(ted);
			demo_SetButtonData(HELPBTN, (VOID *) ted);
			sed_Go(ted);
			demo_SetButtonData(HELPBTN, NULL);
			sed_Close(ted);
			fclose(fp);
			return(TRUE);
		}
	}

	return(FALSE);
}

/* ------------------------------------------------------------------------ */
void demo_FileMissing(char *filename)
/*
	Put up a file missing message.
*/
{
	sed_type sed;

	if ((sed = demo_LoadSed("FileMissing", DL_CENTER | DL_MEMOUT)) != NULL) {

		/* copy filename into field for display */
		sed_SetNameVarValue(sed, "filename", (VOID *) filename);

		sed_Repaint(sed);
		sed_Go(sed);
		sed_Close(sed);
	}
}

/* ------------------------------------------------------------------------ */
char *demo_Text(char *msgtextname)
/*
	gets the message text associated with the given field name; the empty
	string if not found.

	relies on the message text sed in the global demodata structure.
*/
{
	int		fldno;
	char 	*msg = "";

 	if (msgtextname != NULL && *msgtextname != '\0' && demo_GetText() != NULL) {
		if ((fldno = sed_GetNameNo(demo_GetText(), msgtextname)) >= 0) {
			msg = (char *) sed_GetMerge(demo_GetText(), fldno);
		}
	}
	return(msg);
}

/* ------------------------------------------------------------------------ */
opoint  *demo_ClipMoveToParent(win_type win, opoint *p)
/*
*/
{
	opbox 		box;
	win_type	parent_win;

	if ((parent_win = bob_GetParent(win)) != NULL ) {
	
		box.xmin = win_GetXmin(parent_win);	
		box.ymin = win_GetYmin(parent_win);
		box.xmax = win_GetXmax(parent_win) - 1;
		box.ymax = win_GetYmax(parent_win) - 1;
		
		demo_ClipMove(win, p, &box);
	}
	else {
	 	demo_ClipMoveToDisplay(win, p);
	}

 	return(p);
}

/* ------------------------------------------------------------------------ */
opoint  *demo_ClipMoveToDisplay(win_type win, opoint *p)
/*
*/
{
	opbox box;
	
	box.xmin = 0;
	box.ymin = 0;
	box.xmax = disp_GetPixWidth()  - 1;
	box.ymax = disp_GetPixHeight() - 1;

	demo_ClipMove(win, p, &box);

 	return(p);
}

/* ------------------------------------------------------------------------ */
opoint  *demo_ClipMove(win_type win, opoint *p, opbox *clipboxp)
/*
	modify the point p, as the upper-left pix position of the window 
	(sans border), so the the window does not move outside the area indicated
	by the parameter clipboxp.

	intended for use by aux functions processing the WINA_MOVE message.
*/
{

	opbox		*inboxp;

	/*	"clip" point to the left and top--use inbox of win so that the win's
	 *	border does not lie outside the clip area.
	 *
	 *	inbox xmin/ymin is an offset from our border (if any) to the window.
	 */

	inboxp = win_inboxp(win);

	p->x = int_max(p->x, clipboxp->xmin + inboxp->xmin);
	p->y = int_max(p->y, clipboxp->ymin + inboxp->ymin);

	/* 	"clip" point to the bottom and right--use the dimensions of win with 
	 *	its border so that the border is visible.
	 */

	p->x = int_min(p->x, clipboxp->xmax - (bord_GetPixWidth(win) -  1) + inboxp->xmin);
	p->y = int_min(p->y, clipboxp->ymax - (bord_GetPixHeight(win) - 1) + inboxp->ymin);

 	return(p);
}

/* ------------------------------------------------------------------------ */
int aux_Data1(sed_type sed, int msg, VOID *indata, VOID *outdata)
/*
	An auxiliary function for the data1 sed.
	
	Initializes a couple of protected fields and tops the 
	window when gone upon.
*/
{
	int 	fldno;
	int		total_fldno, unitcost_fldno, quantity_fldno;
	static 	purchase_number = 800;

	switch(msg) {
	case AUX_START:
		/*	every time we are loaded up, bump the order number */
		purchase_number++;
		break;
		
	case SED_PRESENTER:
		/*	initialize the purchase date and number */
		if ((fldno = sed_GetNameNo(sed, "purchase_date")) >= 0) {
			tm_Now((struct tm *) sed_GetVar(sed, fldno));
		}	
		if ((fldno = sed_GetNameNo(sed, "purchase_number")) >= 0) {
			*(int *) sed_GetVar(sed, fldno) = purchase_number;
		}	
		return(1);

	case SED_POSTFEXIT:
		if (*sed_GetFieldName(sed, fldno = sed_GetFieldNo(sed)) == '$') {
			/*	leaving one of the inter-dependent money fields, 
			 *	update the total 
			 */	
			quantity_fldno 	= sed_GetNameNo(sed, "$quantity");
			total_fldno 	= sed_GetNameNo(sed, "$total");
			unitcost_fldno 	= sed_GetNameNo(sed, "$unit_cost");

			sed_DoFieldSexit(sed, quantity_fldno);
			sed_DoFieldSexit(sed, unitcost_fldno);

			*(long *) sed_GetVar(sed, total_fldno) =
				(*(long *) sed_GetVar(sed, unitcost_fldno) *  
				(long) *(int *) sed_GetVar(sed, quantity_fldno));

			sed_RepaintField(sed, total_fldno);
		}
		break;

	default:
		break;
	}
	return(aux_Top(sed, msg, indata, outdata));
}

/* ------------------------------------------------------------------------ */
int aux_FileList(sed_type sed, int msg, VOID *indata, VOID *outdata)
/*
	An auxiliary function for the FileList sed.
	
	Way cool updating of the wording ("left" or "right") of the FileMsg
	sed, reflecting the relative positions of the two windows.
*/
{
	sed_type filemsg;
	int 	 fmid, smid, fieldno;

	oak_notused(indata);
	oak_notused(outdata);

	switch(msg) {
	case WINA_STARTGO:
	case WINA_MOVE:
		/* update the left/right field in the filemsg box */
		if ((filemsg = sed_Find("FileMsg")) != NULL) {

			if ((fieldno = sed_GetNameNo(filemsg, "direction")) >= 0) {

				/* get midpoints of the two windows */
				fmid = win_GetLeftCol(filemsg) + sed_GetWidth(filemsg) /2;
				smid = win_GetLeftCol(sed) + sed_GetWidth(sed) /2;
				 
				if (smid < fmid) {
					/* sed is to the left of filemsg */
					sed_SetRecord(filemsg, demo_Text("left"), fieldno);
					
				}
				else {
					/* sed is to the right of filemsg */
					sed_SetRecord(filemsg, demo_Text("right"), fieldno);
				}
				sed_UpdateField(filemsg, fieldno);
			}
		}
		break;
	}

	return(1);	
}

/* ------------------------------------------------------------------------ */
boolean spc_HelpButton(sed_type sed, int scancode)
/*
	special function for the Help "button" sed.
	calls help for the sed to which the "button" sed's data pointer points.
*/
{
	sed_type helpsed;

	oak_notused(sed);

	switch (scancode) {
	case ENTER:

	case MOU_CLICK:
		if ((helpsed = (sed_type) demo_GetButtonData(HELPBTN)) != NULL) {
			help_Show(sed_GetLabel(helpsed), sed_GetFieldNo(helpsed) + 1);
		}
		return(TRUE);

	default:
		break;
	}

	return(FALSE);
}
/* ------------------------------------------------------------------------ */
boolean spc_MoveMode(sed_type sed, int scancode)
/*
	This special key handling function controls the move mode mechanism.
	This allows users to move windows around with the arrow keys.
*/
{
	int 	row, col;

	sed_GetPosition(sed, &row, &col);

	switch(scancode) {
	case UP:
		if (sed_DecField(sed) == SED_STUCK) {
			sed_SetPosition(sed, --row, col);
		}
		return(TRUE);
	case DOWN:
		if (sed_IncField(sed) == SED_STUCK) {
			sed_SetPosition(sed, ++row, col);
		}
		return(TRUE);
	case LEFT:
		sed_SetPosition(sed, row, --col);
		return(TRUE);
	case RIGHT:
		sed_SetPosition(sed, row, ++col);
		return(TRUE);

	default:
		/* ignore all other keystrokes */
		break;
	}

	return(FALSE);
}

/* ------------------------------------------------------------------------ */
void demo_ShowButton(int btn)
/*
	btn is HELPBTN or ESCBTN
*/
{
	if (win_IsEmployed(demo_GetButton(btn))) {
		win_Paint(demo_GetButton(btn));
	}
	else {
		win_Employ(demo_GetButton(btn));
	}
}

/* ------------------------------------------------------------------------ */
void demo_HideButton(int btn)
/*
	btn is HELPBTN or ESCBTN
*/
{
	win_UnEmploy(demo_GetButton(btn));
}

/* ------------------------------------------------------------------------ */
void demo_MemOut(char *msg)
/*
 *	Uses memory cache to display an out-of-memory popup with 
 *	the given message.  Restores the cache after use.
 */
{
	if (demo_FreeMemCache()) {
		oak_ClearAllocErrorTag();
		opc_Prompt(NULL, "C-scape Demo", 0, msg);
		if (oak_GetAllocErrorTag() != 0) {
			/*	
			 *	not even enough room for message.
		 	 *	(may not have been able to re-allocate the memory cache
			 *	after the last memory out situation).
			 *	clear the tag so it does not confuse anyone else.
			 */
			oak_ClearAllocErrorTag();
		}
		demo_AllocMemCache();
	}
}
/* ------------------------------------------------------------------------ */
boolean demo_FreeMemCache()
{
	if (demo_GetMemCache() != NULL) {
	 	ofree(OA_DEMOMEMCACHE, (VOID *)demo_GetMemCache());
		demo_SetMemCache(NULL);
		return(TRUE);
	}
	return(FALSE);
}

/* ------------------------------------------------------------------------ */
boolean demo_AllocMemCache()
{
	VOID *m = NULL;

	demo_SetMemCache(m = omalloc(OA_DEMOMEMCACHE, DEMO_MEMCACHESIZE));
	return(m == NULL ? FALSE : TRUE);
}
/* ------------------------------------------------------------------------ */

int aux_NoControl(win_type win, int msg, VOID *indata, VOID *outdata)
/*
 * 	this auxiliary function ignores all mouse clicks in the border.
 *	it is intended to serve field-less seds with CUA style borders, pre-empting
 *	the border's attempt to grab control.
 */
{
	switch (msg) {
	case WINA_BORDERCLICK:
		*(int *)outdata = 0;
		return(0);
	default:
		break;
	}
 	return(aux_WinDefault(win, msg, indata, outdata));
}
/* ------------------------------------------------------------------------ */

