/*
	csdemo.h

	jmd 9/90

	% header file for the C-scape Demo.

	C-scape 4.0 Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	01/17/91 pmcm	added text only/graphics only stuff
	01/17/91 pmcm	added a bunch of externs for new functions, buttons,
					changed demodata structure and macros
	01/22/91 pmcm	added clock
	01/24/91 jql	changed DEMO_POPUP from 10 to 22 (for help)
	 2/07/91 pmcm	added usermono, mono flags, demo_SetMono
	01/13/92 pmcm	removed restriction on text-and-graphics compilation;
					added DEMO_MODESWITCH
	06/15/92 pmcm	added protos for memory cache and memout popups;
					menu demos return boolean now; added aux_NoControl extern.
	 6/16/92 pmcm	added CLEANCLOSE under DOS.
	 8/04/92 pmcm	added some externs for re-arranged source.
	 8/05/92 pmcm	removed TEXTONLY, GRAFONLY
	 8/22/92 atb	added def for memmove under unix
*/

/* common include files */

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <time.h>

#include "cscape.h"
#include "ostdlib.h"

#include "sfile.h"			/* for screen file stuff */
#include "scancode.h"
/* -------------------------------------------------------------------------- */

#ifdef OAK_X
	/*	 cannot switch to text mode */
#	define DEMO_NOMODESWITCH
#endif
#ifdef OAK_UNIX
#	ifndef memmove
#	       define memmove omemmove
#	endif
#endif
#ifdef OAK_DOS
	/*	do this under dos so that we restore the proper mode when 
 	 *	exiting under a different display mode than we began.
	 *	do it under any systems that need us to explicitly free all
	 *	that we have allocated.
	 */
#	define DEMO_CLEANCLOSE
#endif
/* -------------------------------------------------------------------------- */

/* demo data */

typedef struct _scrn_struct {
	sed_type	text;			/* the sed in which messages are stored */
  	sed_type	menubar;		/* the main menubar screen */
	sed_type	promptbar;		/* the promptbar screen */
	sed_type	back;			/* the background screen */
	win_type	buttons[2];		/* the esc and help buttons */
} scrn_struct;

struct demodata_struct {
	sfile_type		demo_sfile;	 	/* the demo screen file */

	scrn_struct    *currscrns;		/* the current main screens (menubar,
									 *	promptbar, buttons)
 									 */

	scrn_struct    *textscrns;   	/* the text 	mode screens */
	scrn_struct    *grafscrns;   	/* the graphics mode screens */
	
	sed_type		help;			/* current sed for help button help */

	VOID	   	   *textdisp;	 	/* text mode Device Interface */
 	VOID 	   	   *grafdisp;		/* graphics mode Device Interface */

	boolean 		gcapable;		/* can we do graphics? */
	boolean 		textmode;		/* are we in text mode? */
	boolean			usermono;		/* did the user request monochrome? */
	boolean			mono;			/* are we mono because of hardware? */
	int				opmode;			/* operating mode */

	int				graflibcode;	/* graflib initialization code 
									 * 0=uninitialized
									 */

	VOID		   *memcache;		/* memory stash for out-of-memory popup */
};

#define demo_SetMemCache(m)		(demodata.memcache = (m))
#define demo_GetMemCache()		(demodata.memcache)

#define demo_SetCurrScrns(s)	(demodata.currscrns = (s))
#define demo_GetCurrScrns()		(demodata.currscrns)

#define demo_SetTextScrns(s)	(demodata.textscrns = (s))
#define demo_GetTextScrns()		(demodata.textscrns)
#define demo_SetGrafScrns(s)	(demodata.grafscrns = (s))
#define demo_GetGrafScrns()		(demodata.grafscrns)

#define demo_SetButton(btn,win) (demodata.currscrns->buttons[(btn)] = (win))
#define demo_GetButton(btn)		(demodata.currscrns->buttons[(btn)])

#define demo_SetText(sed)		(demodata.currscrns->text = (sed))
#define demo_GetText()			(demodata.currscrns->text)

#define demo_SetBack(sed)		(demodata.currscrns->back = (sed))
#define demo_GetBack()			(demodata.currscrns->back)

#define demo_SetMenuBar(sed)	(demodata.currscrns->menubar = (sed))
#define demo_GetMenuBar()		(demodata.currscrns->menubar)

#define demo_SetPromptBar(sed)	(demodata.currscrns->promptbar = (sed))
#define demo_GetPromptBar()		(demodata.currscrns->promptbar)

#define demo_IsGraphicsCapable()	(demodata.gcapable)
#define demo_IsTextMode()			(demodata.textmode)
#define demo_GetMode()				(demodata.opmode)
#define demo_SetGrafLibCode(x)		(demodata.graflibcode = (x))
#define demo_GetGrafLibCode()		(demodata.graflibcode)

/* operating modes */
#define	MODE_MENU		1		/* we're in the main menu */
#define	MODE_DEMO		2		/* we're demonstrating something */

/*	button id's (index into currscrns.buttons[]) */
#define	HELPBTN			0
#define ESCBTN			1

/*	button function macros */
#define demo_SetButtonData(btn, d)	(win_SetData(demo_GetButton((btn)), (d)))
#define demo_GetButtonData(btn)		(win_GetData(demo_GetButton((btn))))

#define demo_IsVisibleButton(btn)	(win_IsEmployed(demo_GetButton((btn))))

/* demo application memory allocation tags */
#define DEMO_MEMCACHESIZE	1536
#define OA_DEMOMEMCACHE		OA_USER(0)

/* demo message values */

#define DEMO_HYPERFUN	1
#define DEMO_FUNCS		2
#define DEMO_SOURCE		3
#define DEMO_ABOUT		4

#define DEMO_WINDOWS	5
#define DEMO_BORDERS	6

#define DEMO_DATA1		7
#define DEMO_CARDFILE	8		/*	(for later use) */
#define DEMO_STOCK		9

#define DEMO_POPUP		10
#define DEMO_PULLDOWN	11
#define DEMO_123		12
#define DEMO_SLUG		13

#define DEMO_WORLD		14
#define DEMO_TICTAC		15
#define DEMO_HELP		16

#define DEMO_TEXT		18
#define DEMO_GRAPHICS	19

#define DEMO_SYSTEMS	20

#define DEMO_GENERAL	22

#define DEMO_CLOCK		23

#define DEMO_QUIT		444
#define DEMO_NOT		222

/* flags for demo_LoadSed */

#define DL_PAINT		0x0001
#define DL_GO	 		0x0002
#define DL_CLOSE		0x0004
#define DL_CENTER		0x0008
#define DL_HCENTER		0x0010
#define DL_SIZE			0x0020
#define DL_ESCBTN		0x0040
#define DL_HELPBTN		0x0080
#define DL_MEMOUT		0x0100

/*	demo initialization returns */

#define DEMO_NOFAIL				0
#define DEMO_INITFAIL			1
#define DEMO_SFILEOPENFAIL		2
#define DEMO_SFILELOADFAIL		3
#define DEMO_MEMCACHEFAIL		4

/* -------------------------------------------------------------------------- */
/*	external data global to the demo */

/*	CSDEMO.C */
OEXTERN struct demodata_struct demodata;
OEXTERN char *SFILELOADFAIL_MSG;

/* -------------------------------------------------------------------------- */
/* function prototypes */

/* CSCLOK */
extern boolean 	demo_Clock(void);

extern spc_func (spc_ClockMsg);
extern aux_func (aux_ClockMsg);

/* CSDEMO.C */
extern char	   *demo_Text(char *msgtextname);
extern int	 	demo_Query(char *msg);
extern sed_type	demo_LoadSed(char *name, unsigned mode);
extern void 	demo_SetMode(int _mode);
extern boolean 	demo_ViewFile(char *filename);
extern void		demo_FileMissing(char *filename);
extern boolean	demo_SwitchToGraphicsMode(void);
extern boolean 	demo_SwitchToTextMode(void);
extern opoint  *demo_ClipMove(win_type win, opoint *p, opbox *clipboxp);
extern opoint  *demo_ClipMoveToParent(win_type win, opoint *p);
extern opoint  *demo_ClipMoveToDisplay(win_type win, opoint *p);

extern void		demo_ShowButton(int btn);
extern void		demo_HideButton(int btn);

extern boolean demo_AllocMemCache();
extern boolean demo_FreeMemCache();
extern void demo_MemOut(char *msg);

void			demo_Close(void);
scrn_struct    *demo_OpenScrns(void);
void			demo_CloseScrns(void);	

aux_func		(aux_FileList);
aux_func		(aux_Data1);
extern aux_func	(aux_NoControl);

extern spc_func (spc_MoveMode);
extern spc_func (spc_HelpButton);

/* CSWRLD.C */
extern boolean 	demo_World(void);
extern aux_func (aux_List);
extern spc_func (spc_List);

/* CSSTOK.C */
extern boolean 	demo_Stock(void);
extern spc_func (spc_BuySellBox);
extern aux_func (aux_BuySellBox);
extern aux_func (aux_CashBox);
extern spc_func (spc_PriceBox);
extern aux_func (aux_PriceBox);

/* CSMENU.C */
extern boolean	demo_Popup(void);
extern boolean	demo_Pulldown(void);
extern boolean	demo_Slug(int mode);

extern int 	bdwalk(VOID *sdata, int idata);
extern int 	vermont(VOID *sdata, int idata);
extern int 	rent(VOID *sdata, int idata);
extern int 	age(VOID *sdata, int idata);
extern int 	robot(VOID *sdata, int idata);

/* CSBORD.C */
extern boolean 	demo_Borders(void);
extern aux_func (aux_bdcuaDemo);
extern spc_func (spc_bdcuaDemo);

/* CSWIND.C */
extern boolean 	demo_Windows(void);
extern spc_func (spc_Window);
extern aux_func (aux_Window);

/* CSTTAC.C */
extern boolean 	demo_TicTac(void);

/* CSINIT.C */
extern int	 	demo_Open(void);
extern void 	demo_Close(void);
extern boolean  demo_SetMono(void);
/* -------------------------------------------------------------------------- */

