 /*
	csmenu.c

	jmd 9/90

	% menuing system portion of the C-scape Demo.

	C-scape 4.0	Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	Revision History:
	-----------------
	01/17/91 pmcm	sfile'd some stuff
	01/17/91 pmcm	added shadows to slug
	01/28/91 pmcm	hcenter'd msg boxes and slug
	  4/3/91 bkd	added "void" to empty argument lists

	01/02/92 pmcm	bd_cua'd
	 6/15/92 pmcm	added memory checks.
*/

#include "csdemo.h"

#include "msys.h"				/* for the menuing systems */
#include "popdecl.h"			/* for the popups */

/***-------------------------- Local Function Declaration Prototypes -------***/

/***-------------------------- Logicals ------------------------------------***/

/* 	labels for help */

#define DEMO_POPUP_LABEL	22
#define DEMO_PULLDOWN_LABEL	22
#define DEMO_123_LABEL		22
#define DEMO_SLUG_LABEL		22

/* -------------------------------------------------------------------------- */

/* list for pop_Menu--gets initialized by demo_Popup() from sfile */
static char *pop[] = {
	NULL,				/*	Africa 			*/
	NULL,				/*	Antarctica 		*/
	NULL,				/*	Asia 			*/
	NULL,				/*	Australia 		*/
	NULL,				/*  North America 	*/
	NULL,				/*  South America 	*/
	NULL,				/*	Europe 			*/
	NULL
};

/* -------------------------------------------------------------------------- */
boolean demo_Popup(void)
/*
 *	This is demo of the popup menu function.
 */
{
	char 	 	msg[80];
	int 	 	choice = 0;
	sed_type 	msgsed;

	/* put up the message box */
   if ((msgsed = demo_LoadSed("PopupMsg", DL_PAINT)) == NULL) {
		return(FALSE);
	}

	/* 	init the list of menu choices for the pop up */
	pop[0] = demo_Text("africa");
	pop[1] = demo_Text("antarctica");
	pop[2] = demo_Text("asia");
	pop[3] = demo_Text("australia");
	pop[4] = demo_Text("north_america");
	pop[5] = demo_Text("south_america");
	pop[6] = demo_Text("europe");

	/*	clear the error tag so now we can tell later whether 
	 *	pop_Menu had enough room to run
	 */
	oak_ClearAllocErrorTag();

	/* 	create and activate the popup menu */
	choice = pop_Menu(demo_Text("popup_title"), pop, 4, -1, -1, -1, 0x47, DEMO_POPUP_LABEL, bd_cua);

	if (oak_GetAllocErrorTag() != 0) {
		oak_ClearAllocErrorTag();
		demo_MemOut(demo_Text("memout"));
	}

	if (choice > 0) {
		sprintf(msg, "\n %s %s.\n", demo_Text("chose_msg"), pop[choice-1]);

		/*	(add 2 to the requested width 'cuz it includes the border width) */
		pop_Prompt(msg, 5, -1, -1, strlen(msg)+2, 0x47, bd_cua);

		if (oak_GetAllocErrorTag() != 0) {
			oak_ClearAllocErrorTag();
			demo_MemOut(demo_Text("memout"));
		}
	}

	/* 	close the message box */
	if (msgsed != NULL) {
		sed_Close(msgsed);
	}

	return(TRUE);
}

/* -------------------------------------------------------------------------- */
boolean demo_Pulldown(void)
/*
	This is a demo of the framer menu system.
*/
{
	sed_type pulldown, msgsed;

	/* put up the message box */
	if ((msgsed = demo_LoadSed("PulldownMsg", DL_HCENTER | DL_PAINT)) == NULL) {
		return(FALSE);
	}

    if ((pulldown = demo_LoadSed("Pulldown", 0)) == NULL) {
		sed_Close(msgsed);
		return(FALSE);
	}

	sed_SetWidth(pulldown, disp_GetWidth());

	sed_Repaint(pulldown);

	frame_Go(pulldown, ' ', NULL);

	sed_Close(pulldown);

	/* close the message box */
	if (msgsed != NULL) {
		sed_Close(msgsed);
	}

	return(TRUE);
}

/* -------------------------------------------------------------------------- */
boolean demo_Slug(int mode)
/*
	This is a demo of the slug menuing system.
	The 'mode' parameter is used to indicate whether to create
	a vertical (slug) or horizontal (123) menu.
*/
{
	sed_type 	slug, msgsed;
	int 	  	row, col;
	boolean		ret = TRUE;

	if (mode == MENU_HORIZONTAL) {

	    if ((slug = demo_LoadSed("123", 0)) != NULL) {
			sed_SetWidth(slug, disp_GetWidth());
		}

		/* put up the message box */
		msgsed = demo_LoadSed("123msg", DL_HCENTER | DL_PAINT);
	}
	else {
	  	slug = demo_LoadSed("Slug", DL_HCENTER);

		/* put up the message box */
		msgsed = demo_LoadSed("SlugMsg", DL_HCENTER | DL_PAINT);
	}

	if (msgsed == NULL || slug == NULL) {
		ret = FALSE;
	}
	else {
		sed_GetPosition(slug, &row, &col);
		slug_Go(slug, 0, row, col, NULL);
	}

	if (slug != NULL) {
		slug_Close(slug);
	}
	if (msgsed != NULL) {
		sed_Close(msgsed);
	}

	return(ret);
}

/* -------------------------------------------------------------------------- */
int age(VOID *sdata, int idata)
/*
	This is a sample user function.

	It tells when the selected age started.  idata contains the
	time in millions of years.
*/
{
	sed_type sed;

	oak_notused(sdata);

	if ((sed = demo_LoadSed("EraBox", DL_HCENTER | DL_MEMOUT)) == NULL) {
		return(0);
	}

	/* set the value of the years field */
	sed_SetNameVarValue(sed, "years", (VOID *) &idata);

	sed_Repaint(sed);
	sed_Go(sed);
	sed_Close(sed);

	return(0);
}

/* -------------------------------------------------------------------------- */
int robot(VOID *sdata, int idata)
/*
	This is a sample user function.
	It displays a message about the robozoic age.
*/
{
	oak_notused(sdata);
	oak_notused(idata);

	demo_LoadSed("Robozoic", DL_HCENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
	return(0);
}

/* -------------------------------------------------------------------------- */
int bdwalk(VOID *sdata, int idata)
/*
	This is yet another sample user function.

	Loads and shows the Boardwalk message.
*/
{
	oak_notused(sdata);
	oak_notused(idata);

	demo_LoadSed("Boardwalk", DL_HCENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
	return(0);
}

/* -------------------------------------------------------------------------- */
int vermont(VOID *sdata, int idata)
/*
	This is yet another sample user function.

	Loads and shows the Vermont message.
*/
{
	oak_notused(sdata);
	oak_notused(idata);

	demo_LoadSed("Vermont", DL_HCENTER | DL_PAINT | DL_GO | DL_CLOSE | DL_MEMOUT);
	return(0);
}

/* -------------------------------------------------------------------------- */
int rent(VOID *sdata, int idata)
/*
	This is another sample user function.
	It tells the rent for the specified property.
	idata contains the rent.
*/
{
	char msg[80];

	oak_notused(sdata);

	sprintf(msg, " %s%d.\n", demo_Text("rent_msg"), idata);
	oak_ClearAllocErrorTag();

	/*	(add 2 to the requested width 'cuz it includes the border width) */
	pop_Prompt(msg, 9, -1, -1, strlen(msg)+2, 0x47, bd_cua);

	if (oak_GetAllocErrorTag() != 0) {
		oak_ClearAllocErrorTag();
	 	demo_MemOut(demo_Text("memout"));
	}
	return(0);
}
/* -------------------------------------------------------------------------- */


