/*
	csttac.c

	jmd 7/90

	% tic-tac-toe portion of C-scape Demo.

	C-scape 4.0 Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	OVERVIEW:
	---------

	Revision History:
	-----------------
	 7/27/90 mla	fixed diagonal victory
	 7/27/90 mla	added machine as opponent feature
	 7/29/90 jmd	removed whoseturn stuff, assumes human v. machine 
	 8/14/90 jmd	added running score tally
	11/26/90 jmd	moved to demo program
	01/23/91 pmcm	added thinking prompt, demo help button support
	01/24/91 jql	added FN1 case in playing window
	01/27/91 pmcm	added ocolmap_Close in case where can't get all pmaps loaded
	01/30/91 pmcm	removed the quit query
	  4/2/91 bkd	adapted to new pmap color method
	  4/3/91 bkd	fixed typo in new color code.
	  4/3/91 bkd	added "void" to empty argument lists
	  7/3/91 bkd	finalized 3.2c format (new pmap method).

	01/07/92 pmcm	opened tictactoe square windows to pmap size;
					spaced 'em in menu according to size.
	 3/06/92 pmcm	changed mouse handlers to winmou_All
	 6/10/92 pmcm	added MOU_NOFIELD to aux of pmwin for clicks
	 6/15/92 pmcm	added error checks; preened.
	 8/22/92 atb	staticized the definitions of
			tic_LoadPmaps  & tic_ClosePmaps
*/

#include "csdemo.h"

#include "olimits.h"
#include "popdecl.h"		/* for pop_Menu */
#include "pmwinobj.h"		/* for pmwin stuff */
#include "helpdecl.h"		/* for help_Show */

/***------------------------ Function Declaration Prototypes -------***/

aux_func	(aux_TicTac);

OSTATIC int 		tic_SetSpot(int row, int col, int whoseturn);
OSTATIC int 		machine_moves(int *row, int *col);
OSTATIC win_type 	ticwin_Open(void);
OSTATIC boolean 	tic_LoadPmaps(void);
OSTATIC void 		tic_ClosePmaps(void);

/***------------------------ Function Macros -----------------------------***/

#ifdef OAK_DOS
#	define 		inc_rand(l,u) ((int)((double)rand() / ((unsigned)RAND_MAX+1) * \
				((u) - (l) + 1) + (l)))
#else
#	define 		inc_rand(l,u) ((int)((double)rand() / ((unsigned)INT_MAX+1) * \
				((u) - (l) + 1) + (l)))
#endif

/***------------------------ Logicals ------------------------------------***/

#define	TICGO_OK			0
#define	TICGO_STALEMATE		1
#define	TICGO_XWINS			2
#define	TICGO_OWINS			3
#define	TICGO_QUIT			4
#define TICGO_INVALID		5

#define TIC_BLANK	0
#define TIC_X		1		/* player */
#define TIC_O		2		/* machine */

/***------------------------- Module Globals -----------------------------***/

static int tic_grid[3][3] = {
	{ TIC_BLANK, TIC_BLANK, TIC_BLANK },
	{ TIC_BLANK, TIC_BLANK, TIC_BLANK },
	{ TIC_BLANK, TIC_BLANK, TIC_BLANK }
};

static pmap_type tic_pmap[3];

static char *pcx_filename[] = {
	"csblnk.pcx",
	"csx.pcx",
	"cso.pcx"
};

/* ------------------------------------------------------------------------ */
boolean demo_TicTac(void)
/*
	Creates the tictac board and passes control to it.
	Returns FALSE if unable to load .pcx files

	The tictac board consists of nine pmap windows arranged three by three.
	Each pmap window displays one of three pmaps: one for blank sqaures,
	one for squares containing an 'X' and one for squares containing an 'O'.

	The pmap windows are embedded as bobs within a parent sed.
*/
{
	menu_type 		menu;
	sed_type  		sed, score_sed;
	win_type  		ticwin[3][3];
	win_type  		win;
	int		  		row, col;
	int		  		xwins = 0, ywins = 0;
	boolean   		done, backout;
	int				wid, hgt;
	char	  		str[15], *msg;

	if (!tic_LoadPmaps()) {
	 	return(FALSE);
	}

	/* create a sed to hold the tic wins */
	if ((menu = menu_Open()) == NULL) {
	 	return(FALSE);
	}

	/*	create the 9 little tic-tac-toe windows */

	for (row = 0; row < 3; row++) {
		for (col = 0; col < 3; col++) {
			ticwin[row][col] = NULL;
		}
	}
	backout = FALSE;
	for (row = 0; row < 3; row++) {
		for (col = 0; col < 3; col++) {

			if ((ticwin[row][col] = ticwin_Open()) == NULL) {
				backout = TRUE;
				break;
			}
	
			/* attach go and mouse functions to the window */
			obj_SetAux(ticwin[row][col], aux_TicTac);
			win_SetMouse(ticwin[row][col], winmou_All);
			win_SetMouseFeature(ticwin[row][col], MOUF_TRACK);

			/* use window name to locate window in the grid */
			sprintf(str, "%d %d", row, col);
			obj_SetName(ticwin[row][col], str);
		}
	}

	/*	cleanup if we could not open all 9 of the little tic-tac-toe windows */
	if (backout) {
		for (row = 0; row < 3; row++) {
			for (col = 0; col < 3; col++) {
				if (ticwin[row][col] != NULL) {
					win_Close(ticwin[row][col]);
				}
			}
		}
		menu_Destroy(menu);
		tic_ClosePmaps();
		return(FALSE);
	}

	/*  to properly space the pmap windows in a sed menu, figure the 
	 *	character coordinate dimensions, rounding coordinates up.
	 */
	win = ticwin[0][0];
	wid = (int) ofont_GetWidth(disp_GetDefFont());	
	hgt = (int) ofont_GetHeight(disp_GetDefFont());
	wid = (int) ((bord_GetPixWidth(win) + wid - 1) / wid);
	hgt = (int) ((bord_GetPixHeight(win) + hgt - 1) / hgt);

	for (row = 0; row < 3; row++) {
		for (col = 0; col < 3; col++) {

			/* embed the window in the parent menu */
			if (!menu_Printf(menu, "@p[%d,%d]@fb[]", row * hgt, col * wid,
				NULL, &bob_funcs, ticwin[row][col])) {

				/*	(ticwins go with it, too, now that they are bobs) */
				menu_Destroy(menu);
				tic_ClosePmaps();
				return(FALSE);
			}
		}
	}

	/* create the sed */
	if ((sed = sed_Open(menu)) == NULL) {
		menu_Destroy(menu);
		tic_ClosePmaps();
		return(FALSE);
	}

	if (!sed_SetBorder(sed, bd_cua)) {
		sed_Close(sed);
		tic_ClosePmaps();
		return(FALSE);
	}
	sed_SetBorderFeature(sed, BD_PROMPT | BD_MOVE | BD_OUTLINE);

	sed_SetPosition(sed, 3, 43);
	sed_SetShadowAttr(sed, 0x08);
	sed_SetShadow(sed, 1);

	/* load the score board sed from the screen file */
	if ((score_sed = demo_LoadSed("TicTacScore", 0)) == NULL) {
		sed_Close(sed);
		tic_ClosePmaps();
		return(FALSE);
	}

	/* Generate bobbed pmap window */
	if ((win = ticwin_Open()) == NULL) {
		sed_Close(score_sed);
		sed_Close(sed);
		tic_ClosePmaps();
		return(FALSE);
	}
	pmwin_SetPmap(win, tic_pmap[TIC_X]);

	/* attach the pmap window to the score sed */
	if (!menu_Printf(sed_GetMenu(score_sed), "@p[1,29]@fb[]",  
		NULL, &bob_funcs, win)) {
	 	
		win_Close(win);
		sed_Close(score_sed);
		sed_Close(sed);
		tic_ClosePmaps();
		return(FALSE);
	}

	/* point the score field's variable to our local variables */
	sed_SetNameVar(score_sed, "xwins", (VOID *) &xwins);
	sed_SetNameVar(score_sed, "ywins", (VOID *) &ywins);

	sed_Repaint(score_sed);

	demo_SetButtonData(HELPBTN, (VOID *) score_sed);
	demo_ShowButton(HELPBTN);
	demo_ShowButton(ESCBTN);

	done = FALSE;
	while (!done) {
		/* update the tictac board */
		sed_Repaint(sed);

		switch(sed_Go(sed)) {
		case TICGO_STALEMATE:
			msg = demo_Text("stalemate_prompt");
			break;
		case TICGO_XWINS:
			xwins++;
			sed_RepaintField(score_sed, sed_GetNameNo(score_sed, "xwins"));
			msg = demo_Text("you_win_prompt");
			break;
		case TICGO_OWINS:
			ywins++;
			sed_RepaintField(score_sed, sed_GetNameNo(score_sed, "ywins"));
			msg = demo_Text("you_lose_prompt");
			break;
		default:
			done = TRUE;
			break;
		}

		if (!done && demo_Query(msg) == 2) {
			/* 	leave the game */
			done = TRUE;
		}

		/* 	reset the game */
		for (row = 0; row < 3; row++) {
			for (col = 0; col < 3; col++) {
				pmwin_SetPmap(ticwin[row][col], tic_pmap[TIC_BLANK]);
				tic_grid[row][col] = TIC_BLANK;
			}
		}
	}

	demo_HideButton(ESCBTN);
	demo_HideButton(HELPBTN);
	demo_SetButtonData(HELPBTN, NULL);

	/* close the seds (the pmwins are bobs and get closed automatically) */
	sed_Close(sed);
	sed_Close(score_sed);

	/*	pmaps must be closed explicitly */
	tic_ClosePmaps();

	return(TRUE);
}

/* ------------------------------------------------------------------------ */
static win_type ticwin_Open(void)
/*
 *	create a pixel map (pmap) window for displaying 
 *	x/o/blank tic-tac-toe markers (pmaps).
 *
 *	returns a pointer to the window.
 */ 
{
	win_type win;
	opbox 	 pbox;

	/* first define the size of the windows in a opbox structure: */
	pbox.xmin = 0;
	pbox.ymin = 0;
	pbox.xmax = pmap_GetWidth(tic_pmap[TIC_BLANK]);
	pbox.ymax = pmap_GetHeight(tic_pmap[TIC_BLANK]);

	/* 
	 *	create the window with win_PixOpen.  We pass win_PixOpen
	 * 	the class function for the type of window we wish to create,
	 * 	in this case pmwin_Class, a pointer to the opbox structure
	 * 	defining its initial size and position; and the default display
	 *	font.
 	 */	
		
	win = win_PixOpen(pmwin_Class, &pbox, disp_GetDefFont());

	win_SetAttr(win, 0xf3);
	win_SetShadowAttr(win, 0x11);

	/* each window initially has the blank tic-tac-toe marker */
	pmwin_SetPmap(win, tic_pmap[TIC_BLANK]);

	return(win);
}

/* -------------------------------------------------------------------------- */
int aux_TicTac(win_type win, int msg, VOID *indata, VOID *outdata)
/*
	The aux function for our tictac windows.
	This function gets sent a WINA_GO message
	when we call win_Go on the pmap window.

	It handles the character and mouse input.
	(similar to a C-scape fkey function)
*/
{
	int			scancode, key, row, col;
	char	   *name;	
	int			state = TICGO_OK;

	oak_notused(indata);
	oak_notused(outdata);

	if (msg == WINA_GO) {

		while(state == TICGO_OK) {

			/* player's move */
			switch ((scancode = kb_Read())) {
			case HELP_KEY:	/* help key for non-DOS machines */
			case FN1:
				/* we're not in a sed, call the help system directly */
				help_Show(12,0);
				continue;

			case MOU_THERE:
				/* pass control to another window (next win has been set by 
				 * the mouse handler) 
				 */
				state = TICGO_QUIT;
				break;

			case ESC:
				/* ESC to quit (the next win is NULL) */
				state = TICGO_QUIT;
				break;

			case MOU_NOFIELD:
			case MOU_CLICK:
				/* change the contents of the window */
				/* use our name to find our row,col position */
				if ((name = obj_GetName(win)) != NULL) {
					sscanf(name, "%d %d", &row, &col);

					state = tic_SetSpot(row, col, TIC_X);
				}
				break;

			case MOU_HERE:					/* ignore */
				continue;

			default:
				/* test for numeric keys */
				key = ascii(scancode);
				if (key >= '1' && key <= '9') {
					/* translate numeric keys to row, col positions */
					switch(key) {
					case '1':
						row = 2;
						col = 0; 
						break;
					case '2':
						row = 2;
						col = 1; 
						break;
					case '3':
						row = 2;
						col = 2; 
						break;
					case '4':
						row = 1;
						col = 0; 
						break;
					case '5':
						row = 1;
						col = 1; 
						break;
					case '6':
						row = 1;
						col = 2; 
						break;
					case '7':
						row = 0;
						col = 0; 
						break;
					case '8':
						row = 0;
						col = 1; 
						break;
					case '9':
						row = 0;
						col = 2; 
						break;
					}

					state = tic_SetSpot(row, col, TIC_X);
				}
				else {
					state = TICGO_INVALID;
				}
				break;
			
			}

			/* computer's move */
			if (state == TICGO_OK) {
				/* pause, make it look like the machine is thinking */
				sed_BorderPrompt(bob_GetParent(win), demo_Text("thinking"));
				hard_Pause(100);
				tone();
				machine_moves(&row, &col);
				state = tic_SetSpot(row, col, TIC_O);
				sed_BorderPrompt(bob_GetParent(win), NULL);
			}

			/* reset invalid state */
			if (state == TICGO_INVALID) {
				state = TICGO_OK;
			}
		}

		/* the return value is returned via win_Go */
		return(state);
	}

	return(1);
}

/* -------------------------------------------------------------------------- */
static int tic_SetSpot(int row, int col, int whoseturn)
/*
	Set a spot in the tictac grid

	Returns  a TICGO_ code
*/
{
	boolean		victory;
	int			ret = TICGO_OK;
	int			r, c;
	char 	    str[20];
	win_type 	win;

	/* figure out which window we are in (use the win names) */
	sprintf(str, "%d %d", row, col);
	win = obj_Find(str);

	if (win == NULL) {
		return(TICGO_OK);
	}

	if (tic_grid[row][col] != TIC_BLANK) {
		tone();
		return(TICGO_INVALID);
	}
	else {
		tic_grid[row][col] = whoseturn;
		pmwin_SetPmap(win, tic_pmap[whoseturn]);

		win_Paint(win);

		/* test for victory: we only need test what just changed */
		/* test the row */
		victory = TRUE;
		for (c = 0; c < 3; c++) {
			if (tic_grid[row][c] != tic_grid[row][col]) {
				/* not a match, no victory this way */
				victory = FALSE;
				break;
			}
		}
		if (victory) {
			return((whoseturn == TIC_X) ? TICGO_XWINS : TICGO_OWINS);
		}

		/* test the column */
		victory = TRUE;
		for (r = 0; r < 3; r++) {
			if (tic_grid[r][col] != tic_grid[row][col]) {
				/* not a match, no victory this way */
				victory = FALSE;
				break;
			}
		}
		if (victory) {
			return((whoseturn == TIC_X) ? TICGO_XWINS : TICGO_OWINS);
		}

		/* test diagonal */
		victory = FALSE;
		if (row == col) {
			if (tic_grid[0][0] == tic_grid[1][1] && tic_grid[1][1] == tic_grid[2][2]) {
				victory = TRUE;
			}
		}
		if (row == 2 - col) {
			if (tic_grid[0][2] == tic_grid[1][1] && tic_grid[1][1] == tic_grid[2][0]) {
				victory = TRUE;
			}
		}

		if (victory) {
			return((whoseturn == TIC_X) ? TICGO_XWINS : TICGO_OWINS);
		}

		/* test for stalemate */
		ret = TICGO_STALEMATE;
		for (r = 0; r < 3; r++) {
			for (c = 0; c < 3; c++) {
				if (tic_grid[r][c] == TIC_BLANK) {
					/* not a stalemate, reset ret */
					ret = TICGO_OK;
				}
			}
		}
	}

	return(ret);
}

/* -------------------------------------------------------------------------- */
static int machine_moves(int *rowp, int *colp)
/*
	generates the machine's move.
	uses global array int tic_grid[3][3] to see board

	Returns the machine's move via rowp and colp.
*/
{
	int r, c, ru, cu, mine, yours, blank;

	/* win me */

	/* by row */
	for (r = 0; r < 3; r++) {
		mine = blank = 0;
		for (c = 0; c < 3; c++) {
			if (tic_grid[r][c] == TIC_O) {
				mine++;
			}
			else if (tic_grid[r][c] == TIC_BLANK) {
				blank++;
				*rowp = r;
				*colp = c;
			}
		}
		if (mine == 2 && blank == 1) {
			return(TICGO_OWINS);
		}
	}

	/* by col */
	for (c = 0; c < 3; c++) {
		mine = blank = 0;
		for (r = 0; r < 3; r++) {
			if (tic_grid[r][c] == TIC_O) {
				mine++;
			}
			else if (tic_grid[r][c] == TIC_BLANK) {
				blank++;
				*rowp = r;
				*colp = c;
			}
		}
		if (mine == 2 && blank == 1) {
			return(TICGO_OWINS);
		}
	}

	/* by diagnal */
	/* NE to SW */
	mine = blank = 0;
	for (r = 0, c = 2; r < 3; r++, c--) {
		if (tic_grid[r][c] == TIC_O) {
			mine++;
		}
		else if (tic_grid[r][c] == TIC_BLANK) {
			blank++;
			*rowp = r;
			*colp = c;
		}
	}
	if (mine == 2 && blank == 1) {
		return(TICGO_OWINS);
	}

	/* NW to SE */
	mine = blank = 0;
	for (r = 0, c = 0; r < 3; r++, c++) {
		if (tic_grid[r][c] == TIC_O) {
			mine++;
		}
		else if (tic_grid[r][c] == TIC_BLANK) {
			blank++;
			*rowp = r;
			*colp = c;
		}
	}
	if (mine == 2 && blank == 1) {
		return(TICGO_OWINS);
	}

	/* block you */

	/* by row */
	for (r = 0; r < 3; r++) {
		yours = blank = 0;
		for (c = 0; c < 3; c++) {
			if (tic_grid[r][c] == TIC_X) {
				yours++;
			}
			else if (tic_grid[r][c] == TIC_BLANK) {
				blank++;
				*rowp = r;
				*colp = c;
			}
		}
		if (yours == 2 && blank == 1) {
			return(TICGO_OK);
		}
	}

	/* by col */
	for (c = 0; c < 3; c++) {
		yours = blank = 0;
		for (r = 0; r < 3; r++) {
			if (tic_grid[r][c] == TIC_X) {
				yours++;
			}
			else if (tic_grid[r][c] == TIC_BLANK) {
				blank++;
				*rowp = r;
				*colp = c;
			}
		}
		if (yours == 2 && blank == 1) {
			return(TICGO_OK);
		}
	}

	/* by diaginal */
	/* NE to SW */
	yours = blank = 0;
	for (r = 0, c = 2; r < 3; r++, c--) {
		if (tic_grid[r][c] == TIC_X) {
			yours++;
		}
		else if (tic_grid[r][c] == TIC_BLANK) {
			blank++;
			*rowp = r;
			*colp = c;
		}
	}
	if (yours == 2 && blank == 1) {
		return(TICGO_OK);
	}

	/* NW to SE */
	yours = blank = 0;
	for (r = 0, c = 0; r < 3; r++, c++) {
		if (tic_grid[r][c] == TIC_X) {
			yours++;
		}
		else if (tic_grid[r][c] == TIC_BLANK) {
			blank++;
			*rowp = r;
			*colp = c;
		}
	}
	if (yours == 2 && blank == 1) {
		return(TICGO_OK);
	}

	/* two on edge strategy */

	if (tic_grid[1][1] == TIC_BLANK) {
		yours = mine = 0;
		for (r = 0; r < 3; r++) {
			for (c = 0; c < 3; c++) {
				if (*rowp != 1 || *colp != 1) {
					if (tic_grid[r][c] == TIC_X) {
						yours++;
					}
					else if (tic_grid[r][c] == TIC_O) {
						mine++;
					}
				}
			}
		}
		if (yours > 1 || mine > 1) {
			*rowp = *colp = 1;
			return(TICGO_OK);
		}
	}

	/* just move to any side or corner (choose randomly)  */

	for (r = inc_rand(0, 2), ru = r + 3; *rowp = r % 3, r < ru; r++) {
		for (c = inc_rand(0, 2), cu = c + 3; *colp = c % 3, c < cu; c++) {
			if (tic_grid[*rowp][*colp] == TIC_BLANK) {
				if (*rowp != 1 || *colp != 1) {
					return(TICGO_OK);
				}
			}
		}
	}

	/* take the center as a last choice */
	if (tic_grid[1][1] == TIC_BLANK) {
		*rowp = *colp = 1;
		return(TICGO_OK);
	}

	/* no moves left */
	return(TICGO_STALEMATE);
}
/* -------------------------------------------------------------------------- */

static boolean tic_LoadPmaps(void)
/*
 *	loads tic-tac-toe board markers from PCX files: "cso.pcx", "csx.pcx",
 *	and "csblank.pcx".
 *
 *	returns FALSE if unable load them all.
 */
{
	int 	 i;
	FILE	*fp;

	for (i = 0; i < 3; i++) {
		tic_pmap[i] = NULL;
	}

	for (i = 0; i < 3; i++) {
		if ((fp = fopen(pcx_filename[i], "rb")) == NULL) {
			demo_FileMissing(pcx_filename[i]);
			goto FAIL;
		}

		/* now, load the pmap from the file and close the file */	
		if ((tic_pmap[i] = pmap_Load(fp, def_PmapFilePCX)) == NULL) {
			goto FAIL;
		}

		/* close the file */
		fclose(fp);

		/* now, dispload the thing */
		if (!pmap_DispLoad(tic_pmap[i])) {
			goto FAIL;
		}
	}
	return(TRUE);

FAIL:
	tic_ClosePmaps();
	return(FALSE);
}
/* -------------------------------------------------------------------------- */

static void tic_ClosePmaps(void)
/*
 *	closes the pixel maps of the tic-tac-toe markers and frees the memory
 *	they use.
 */
{
	int i;

	for (i = 0; i < 3; i++) {
		if (tic_pmap[i] != NULL) {
			pmap_Close(tic_pmap[i]);
			tic_pmap[i] = NULL;
		}
	}
	return;
}
/* -------------------------------------------------------------------------- */

