/*
	cswind.c

	jmd 11/90

	% windows portion of the C-scape Demo.

	C-scape 4.0 Demo Program
	Copyright (c) 1992 by Liant Software Corporation.
	ALL RIGHTS RESERVED.

	Overview:
	---------

	Revision History:
	-----------------
	 1/10/90 pmcm	moved window start position
	 1/11/91 pmcm	added F3 as icon toggle to spc_Icon
	 1/11/91 pmcm	fixed icon position off-by-one error
	 1/12/91 pmcm	added BD_TOP to icon and window borders
	 1/23/91 pmcm	added highlighting to active icon/window
	 1/23/91 pmcm	change mou handler from Track to Click
	 1/24/91 pmcm	made allowance for help, removed FN1 stops in special funcs
	 4/03/91 bkd	added "void" to empty argument lists
	 3/06/92 pmcm	changed mouse handlers to winmou_All
	 6/08/92 pmcm	removed cheezy icon stuff
	 6/15/92 pmcm	added memory checks and cleanup code
	 7/20/92 mla	changed sed_SetName() to obj_SetName()
*/

#include "csdemo.h"
#include "oakalloc.h"	/*	for omalloc */

/***-------------------------- Logicals ------------------------------------***/

#define WINDOW_LABEL 		23
#define WINDOW_COUNT 		6

/***-------------------------- Local Function Declaration Prototypes -------***/

aux_func 	(aux_Window);
spc_func 	(spc_Window);

/***--------------------------- Module Globals -----------------------------***/

/* array of windows and data */

static 		sed_type  		win_array[WINDOW_COUNT];	

typedef struct _windata {
	int			id;				/*  window's place in win_array */
}	windata_struct;

typedef windata_struct *windata_type;

/* -------------------------------------------------------------------------- */
boolean demo_Windows(void)
/*
 *	Create a bunch of windows, let the user move them around.
 */
{
	char 	  		title[80];		   /* buffer for building title string */
	sed_type  		winmsg;
	int 	  		i;
	menu_type 		menu;
	windata_type	wdp;
	boolean			ret = FALSE;

	/* create some simple windows */

	for (i = 0; i < WINDOW_COUNT; i++) {
		win_array[i] = NULL;
	}
	for (i = 0; i < WINDOW_COUNT; i++) {
		/* define fields in the window */

		if ((menu = menu_Open()) == NULL) {
			goto QUIT;
		}
		
		if (!menu_Printf(menu, " @f{row}[%s: ###] \n", NULL, &int_funcs,
			demo_Text("row")) || 
			!menu_Printf(menu, " @f{col}[%s: ###] ", NULL, &int_funcs, 
			demo_Text("col"))) {

			menu_Destroy(menu);
			goto QUIT;
		}

		/* open the window */
		if ((win_array[i] = sed_Open(menu)) == NULL) {
			menu_Destroy(menu);
			goto QUIT;
		}

		sed_SetColors(win_array[i], (byte) (i + 1), (byte) (i + 1), (byte) (i + 0x70));
		sed_SetShadowAttr(win_array[i], 0x08);
		sed_SetLabel(win_array[i], WINDOW_LABEL);
		obj_SetName(win_array[i], "win");
		sed_SetAux(win_array[i], aux_Window);
		sed_SetSpecial(win_array[i], spc_Window);
		sed_SetMouse(win_array[i], winmou_All);

		if ((wdp = (windata_type) omalloc(OA_NOTAG, sizeof(windata_struct))) != NULL) {
			wdp->id = i;
			sed_SetData(win_array[i], (VOID *) wdp);
		}
		else {
			goto QUIT;
		}

		/* allocate space for the fields' variables */
	    if (!sed_Alloc(win_array[i])) {
			goto QUIT;
		}

		if (!sed_SetBorder(win_array[i], bd_cua)) {
			goto QUIT;
		}
		sprintf(title, "%s %d", demo_Text("window"), i);
		sed_SetBorderTitle(win_array[i], title);
		sed_SetBorderColor(win_array[i], 0x07);
		sed_SetBorderFeature(win_array[i], 
			BD_TITLE | BD_PROMPT | BD_MOVE | BD_TOP);

		/* position the window */
		sed_SetPosition(win_array[i], i + 2, (i * 3) + 10);
		sed_SetWidth(win_array[i], 15);
	}

	/* load and display the message window */
	winmsg = demo_LoadSed("WindowMsg", DL_PAINT);

	/*	any window will do for help--all have same label */
	demo_SetButtonData(HELPBTN, (VOID *) win_array[0]);
	demo_ShowButton(HELPBTN);
	demo_ShowButton(ESCBTN);

	for (i = 0; i < WINDOW_COUNT; i++) {
		/* paint the windows on the display */
		sed_Repaint(win_array[i]);
	}

	/* pass control to the last window */
	sed_Go(win_array[WINDOW_COUNT - 1]);

	demo_SetButtonData(HELPBTN, NULL);
	demo_HideButton(HELPBTN);
	demo_HideButton(ESCBTN);

	sed_Close(winmsg);
	ret = TRUE;

QUIT:
	for (i = 0; i < WINDOW_COUNT; i++) {
		if (win_array[i] != NULL) {

			/*	free window data */
			if ((wdp = (windata_type) sed_GetData(win_array[i])) != NULL ) {
				ofree(OA_NOTAG, wdp);
			}

			win_Close(win_array[i]);
		}
	}

	return(ret);
}

/* -------------------------------------------------------------------------- */
boolean spc_Window(sed_type sed, int scancode)
/*
*/
{
	int 	 		row, col, i;
	windata_type	wdp = NULL;

	/* check for move mode keys */
	if (spc_MoveMode(sed, scancode)) {
		return(TRUE);
	}

	wdp = (windata_type) sed_GetData(sed);

	switch (scancode) {
	case TAB:
		/* 	go to next window */
		if (wdp != NULL) {
			i = wdp->id;
			i = (i + 1) % WINDOW_COUNT;
			sed_SetNextWin(sed, win_array[i]);
			sed_ToggleExit(sed);
			return(TRUE);
		}
		break;

	case ENTER:
		/* 	update window position to new entry  */
		sed_DoFieldSexit(sed, sed_GetFieldNo(sed));

		row = *((int *) sed_GetNameVar(sed, "row")) - 1;
		col = *((int *) sed_GetNameVar(sed, "col")) - 1;

		sed_SetPosition(sed, row, col);

		/* 	go to the other field */
		if (sed_IncField(sed) == SED_STUCK) {
			sed_GotoFirstField(sed);
		}
		return(TRUE);

	}
	return(FALSE);
}

/* -------------------------------------------------------------------------- */
int aux_Window(sed_type sed, int msg, VOID *indata, VOID *outdata)
/*
*/
{
	opoint 		   *pntp;
	int		 		row, col;
	windata_type 	wdp = NULL;

	oak_notused(outdata);
	oak_notused(wdp);

	switch(msg) {
	case WINA_STARTGO:
		/* bring window to the foreground when go is called on it */
		sed_Top(sed);

		/*	highlight border to show we are active window.
 		 *	(this only affects the standard borders and the character
		 *	mode version of the CUA style border.  it is ignored by the
		 *	graphics CUA border.)
		 */
		sed_SetBorderColor(sed, 0x0f);
		sed_Repaint(sed);
		break;

	case WINA_ENDGO:
		/*	unhighlight border to show we are no longer active window 
 		 *	(this only affects the standard borders and the character
		 *	mode version of the CUA style border.  it is ignored by the
		 *	graphics CUA border.)
		 */
		sed_SetBorderColor(sed, 0x07);	

		/*	we may be quitting or passing control to another window
		 *	repaint only if we are still on the display.
		 */
		if(win_IsEmployed(sed)) {
		 	sed_Repaint(sed);
		}	
		break;

	case WINA_MOVE:
		if ((wdp = (windata_type) win_GetData(sed)) != NULL) {
   			pntp = (opoint *) indata;

			/* 	don't let the window move off the display */
			demo_ClipMoveToDisplay(sed, pntp);

			/* convert pixel coords to character coords for display */
			row = opcoord_GetYRow(pntp->y, win_GetFont(sed));
			col = opcoord_GetXCol(pntp->x, win_GetFont(sed));
		
			sed_SetNameVarValue(sed, "row", &row);
			sed_SetNameVarValue(sed, "col", &col);
			sed_RepaintFields(sed);
		}
		break;

	default:
		break;
	}	

	return(1);
}
/* -------------------------------------------------------------------------- */
